/* *FH*******************************************************************  */
/* *                                                                    *  */
/* * FILE-NAME     : diat_md.c                                          *  */
/* * FUNCTION      : MD for diatomic in WCA fluid                       *  */
/* *                                                                    *  */
/* * AUTHOR        : Ch. Dellago                                        *  */
/* * CREATION-DATE : 05-02-1998                                         *  */
/* *                                                                    *  */
/* **********************************************************************  */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <signal.h>

#include "odeinty.h"
#include "odeintc.h"
#include "diat_md.h"

#include "Xpgp.h"

#include <sys/times.h>
#include <sys/types.h>

#ifdef GRAPHICS
extern void init_graphics  (REAL  xmin, REAL  xmax, REAL  ymin, REAL ymax);
extern void draw_disks     (REAL  *r, WORD ndisks);
extern void close_graphics (void);
extern void test_graphics (void);
#endif

/*****************************************************************************/
/*      Global variables                                                     */
/*****************************************************************************/

REAL        g_box_size[DIM];              /* box size                        */
REAL        g_box_size_2[DIM];            /* box size / 2                    */
REAL        g_box_volume;                 /* box area                        */
WORD        g_n_disks;                    /* number of disks                 */
WORD        g_n_phas;                     /* dimension of phase space        */
WORD        g_n_conf;                     /* dimension of configuration space*/
static long g_idum;                       /* seed for random number generator*/
REAL        g_rho;                        /* density                         */
REAL       *g_y;                          /* pointer to system               */
REAL       *g_F;                          /* pointer to force                */
REAL        g_stepsize;                   /* step size                       */
REAL        g_ww_radius;                  /* interaction radius              */
REAL        g_ww_radius_2;                /* interaction radius / 2          */
REAL        g_energy;                     /* energy                          */
REAL        g_height;                     /* barrier height                  */
REAL        g_width;                      /* barrier width                   */
REAL        g_state;                      /* state of diatomic               */
REAL        g_R_A;                        /* region A                        */
REAL        g_R_B;                        /* region B                        */

/*****************************************************************************/

double fran (long *idum)
{
    long          k;
    float         ans;

    *idum ^= MASK;
    k = (*idum) / IQ;
    *idum = IA * (*idum - k * IQ) - IR * k;
    if (*idum < 0) *idum += IM;
    ans = AM * (*idum);
    *idum ^= MASK;
    return ((double)(ans));
}

/***************************************************************************/

REAL gauss (REAL  temp,
            long *idum)
{
    REAL  v1;
    REAL  v2;
    REAL  r;
    REAL  fac;

    do
    {
        v1 = 2.0 * fran (idum) - 1.0;
        v2 = 2.0 * fran (idum) - 1.0;

        r = v1 * v1 + v2 * v2;
    } while (r > 1.0);

    fac = sqrt (-2.0 * log (r) / r);

    return (sqrt (temp) * v1 * fac);
}

/***************************************************************************/

#ifdef  AIX
REAL cputime (void)
{
    struct tms time_buff;

    times (&time_buff);

    return ((REAL) time_buff.tms_utime / (REAL) CLK_TCK);
}
#endif

/***************************************************************************/

REAL timer (WORD  opcode)
{
    static unsigned int stime;

    switch (opcode)
    {
        case START:
        {
            stime = clock ();
            return (0.0);
        }
        case STOP:
            return ((REAL) (clock () - stime) / CLOCKS_PER_SEC);

        default:
            printf ("Parameterfehler bei timer !!!\n");
            return (0.0);
    }
}

/***************************************************************************/

char  next_char (FILE *stream)
{
     static WORD  gl_state = CODE;
     char         prev_c   = EOF;
     int          c;

     while ((c = getc (stream)) != EOF)
     {
          switch (gl_state)
          {
             case CODE:
                 if (c == '/')
                    gl_state = TO_COMM;
                 else
                    return (c);
                 break;

             case TO_COMM:
                 if (c == '*')
                    gl_state = COMM;
                 else
                 {
                    ungetc (c, stream);
                    return (prev_c);
                 }
                 break;

             case COMM:
                 if (c == '*')
                    gl_state = TO_CODE;
                 break;

             case TO_CODE:
                 if     (c == '/')
                    gl_state = CODE;
                 else if (c != '*')
                    gl_state = COMM;
                 break;
          }
          prev_c = c;
     }

     return (EOF);
}

/***************************************************************************/

char *read_line (FILE  *stream,
                 char  *string,
                 WORD   length)
{
     char   *cp;
     int     c;

     cp = string;

     for (c = next_char(stream); length > 0 && c != EOF && c != '\n'; )
     {
           *cp++= c;
            c   = next_char (stream);
     }

     if (length == 0)
     {
         printf ("READ_LINE> Line too long !!\n");
         return (NULL);
     }

     *cp = '\0';

     if (c == EOF)
        return (NULL);
     else
        return (string);
}

/***************************************************************************/

char  *str_time (REAL  seconds,
                 char *tstring)
{

      REAL    sec;
      WORD    minutes;
      WORD    hours;

      hours      = ((DWORD) seconds) / 3600;
      minutes    = ((DWORD) seconds) / 60 - hours * 60;
      sec        = seconds - hours * 3600 - minutes * 60;

      sprintf (tstring, "%02d:%02d:%05.2lf", hours, minutes, sec);

      return (tstring);
}

/***************************************************************************/

void  input (char   *filename,
             INPUT_STR  *input_str)
{
      FILE  *stream;
      char   line_buffer[256];
      int    nl;

      if ((stream = fopen (filename, "r")) == NULL)
      {
           printf ("INPUT> Error opening parameter file %s\n", filename);
           exit (0);
      }

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->stepsize));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->nsteps));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->nio));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->nplot));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%d", &(input_str->ndisks));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->rho));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->R_A));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->R_B));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->energy));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->height));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->width));

      /* Schliesse Parameterdatei */
      fclose (stream);
}

/***************************************************************************/


REAL *ralloc (DWORD  size)
{
      REAL  *address;

      if ((address = (REAL *) malloc (sizeof (REAL) * size)) == NULL)
      {
          printf ("Fehler beim allokieren von %ld Bytes!!\n", sizeof (REAL) * size);
          exit (0);
      }

      return (address);
}

/******************************************************************************/

REAL **palloc (DWORD  size)
{
      REAL  **address;

      if ((address = (REAL **) malloc (sizeof (REAL *) * size)) == NULL)
      {
          printf ("Fehler beim allokieren von %ld Bytes!!\n", sizeof (REAL *) * size);
          exit (0);
      }

      return (address);
}


/******************************************************************************/

void  q_image (REAL  *y1,
               REAL  *y2,
               REAL  *dq)
{
    WORD  i;

    for (i = 0; i < DIM; i++)
        if ((dq[i] = y2[i] - y1[i]) >  g_box_size_2[i])
            dq[i] -= g_box_size[i];
        else if (dq[i] <- g_box_size_2[i])
            dq[i] += g_box_size[i];
}

/******************************************************************************/

REAL distance (REAL  *y)
{
    REAL  q[DIM];

    q_image (y+0*DIM, y+1*DIM, q);

    return (sqrt(q[0]*q[0] + q[1]*q[1]));
}

/******************************************************************************/

REAL pot_energy (REAL  *y)
{
    REAL  s;
    WORD  i, j;
    REAL  q[DIM];
    REAL  r;
    REAL r6;
    REAL  z, a, w;

    a = g_ww_radius;
    w = g_width;

    s = 0.0;

    for (i = 0; i < g_n_disks - 1; i++)
    {
        for (j = i + 1; j < g_n_disks; j++)
        {
            q_image (y+i*DIM, y+j*DIM, q);

            r = q[0]*q[0] + q[1]*q[1];

            if (r < g_ww_radius_2)
            {
                r6 = 1.0 / (r * r * r);

                s += (4.0 * r6) * (r6 - 1.0) + 1.0;
            }

            if (i == 0 && j == 1)
            {
                z  = sqrt(r) - w - a;
                s += g_height * (1-z*z/(w*w)) * (1-z*z/(w*w));
            }
        }
    }

    return (s);
}

/******************************************************************************/

REAL kin_energy (REAL  *y)
{
    REAL  s;
    WORD  i;

    s = 0.0;

    for (i = DIM*g_n_disks; i < 2*DIM*g_n_disks; i++)
          s += y[i]*y[i];

    return (s / 2.0);
}

/******************************************************************************/

void tot_mom (REAL  *y,
              REAL  *p,
              WORD   ndisks)
{
    REAL  s;
    WORD  i, j;

    for (j = 0; j < DIM; j++)
        p[j] = 0.0;

    for (i = 0; i < ndisks; i++)
        for (j = 0; j < DIM; j++)
            p[j] += y[DIM * ndisks + i * DIM + j];
}

/***************************************************************************/

REAL tot_energy (REAL *y)
{
    return (pot_energy (y) + kin_energy (y));
}

/***************************************************************************/

void force (REAL *y,
            REAL *F)
{
    WORD i, j;
    WORD ip, jp;
    REAL r6, r, r2, r2i, f, d;
    REAL q[DIM];
    REAL  z, a, w;

    a = g_ww_radius;
    w = g_width;


    /* Initialize forces */
    for (i = 0; i < DIM * g_n_disks; i++)
        F[i]=0.0;

    /* Interaction between particles */
    for (i = 0; i < g_n_disks - 1; i++)
    {
        for (j = i + 1; j < g_n_disks; j++)
        {
            q_image (y+i*DIM, y+j*DIM, q);

            r2 = q[0]*q[0] + q[1]*q[1];

            if (r2 < g_ww_radius_2)
            {
                r6 = r2 * r2 * r2;

                f = 24.0 / (r6 * r2) * (2.0 / r6 - 1.0);

                /* Forces */
                F[i*DIM+0] -= f * q[0];
                F[i*DIM+1] -= f * q[1];

                F[j*DIM+0] += f * q[0];
                F[j*DIM+1] += f * q[1];
            }

            if (i == 0 && j == 1)
            {
                r = sqrt(r2);
                z = r - w - a;

                f = 4.0 * g_height * (1.-z*z/(w*w)) * z / (w*w);

                F[i*DIM+0] -= f * q[0] / r;
                F[i*DIM+1] -= f * q[1] / r;

                F[j*DIM+0] += f * q[0] / r;
                F[j*DIM+1] += f * q[1] / r;
            }
        }
    }
}

/***************************************************************************/

REAL H_A (REAL  *y)
{
    REAL  q[DIM];
    REAL  r, z;

    q_image (y+0*DIM, y+1*DIM, q);

    r = sqrt(q[0]*q[0] + q[1]*q[1]);

    z = r - g_ww_radius - g_width;

    if (z < 0.0)
        return (1.0);
    else
        return (0.0);
}


/***************************************************************************/

REAL h_A (REAL  *y)
{
    REAL  q[DIM];
    REAL  r, z;

    q_image (y+0*DIM, y+1*DIM, q);

    r = sqrt(q[0]*q[0] + q[1]*q[1]);

    if (r < g_R_A)
	return (1.0);
    else
	return (0.0);
}

/***************************************************************************/

REAL h_B (REAL  *y)
{
    REAL  q[DIM];
    REAL  r;

    q_image (y+0*DIM, y+1*DIM, q);

    r = sqrt(q[0]*q[0] + q[1]*q[1]);

    if (r > g_R_B)
	return (1.0);
    else
	return (0.0);
}

/******************************************************************************/

void  remap (REAL *y,
             WORD  ndisks)
{
    WORD i, j;

    for (i = 0; i < ndisks; i++)
    {
        for (j = 0; j < DIM; j++)
        {
            if (y[i*DIM + j] >  g_box_size[j]) y[i*DIM + j] -= g_box_size[j];
            if (y[i*DIM + j] <  0.0)           y[i*DIM + j] += g_box_size[j];
        }
    }
}

/***************************************************************************/

void propagate (REAL *y,
                REAL *F,
                REAL  stepsize,
                WORD  ndisks)
{
    REAL dt;
    DWORD j;

    dt = stepsize;

    /* remap particle */
    remap (y, ndisks);

    /* propagate coordinates */
    for (j = 0; j < g_n_conf; j++)
    {
         y[j] = y[j] + dt * y[g_n_conf + j] + dt * dt * F[j] * 0.5;
         y[g_n_conf + j] = y[g_n_conf + j] + dt * F[j] * 0.5;
    }

    /* Calculate new forces */
    force (y, F);

    /* propagate momenta */
    for (j = 0; j < g_n_conf; j++)
        y[g_n_conf + j] += dt *  F[j] * 0.5;
}

/***************************************************************************/

void  impuls_init (REAL  *y,
                   WORD   ndisks)
{
    WORD   i, j;
    REAL   sum[DIM];
    REAL   s, c;
    REAL   V, K;

    /* initilize */
    for (j = 0; j < DIM; j++)
        sum[j] = 0.0;

    /* momenta from Gauss distribution */
    for (i = 0; i < ndisks; i++)
        for (j = 0; j < DIM; j++) {
            y[DIM*ndisks + DIM*i + j]   =  gauss (1.0,  &g_idum);
            sum[j] += y[DIM*ndisks + DIM*i + j]; }

    /* reset total momentum */
    for (j = 0; j < DIM; j++)
        sum[j] /= ((REAL) ndisks);

    for (i = 0; i < ndisks; i++)
        for (j = 0; j < DIM; j++)
            y[DIM*ndisks + DIM*i + j] -= sum[j];

    /* energy */
    V = pot_energy (y);
    K = kin_energy (y);

    c = (g_energy-V) / K;

    if (c < 0.0)
    {
	printf ("MOM_INIT> V=%lf > E=%lf\n", V, g_energy);
	exit (0);
    }

    c = sqrt(c);

    /* rescale momenta */
    for (i = 0; i < ndisks; i++)
        for (j = 0; j < DIM; j++)
            y[DIM*ndisks + DIM*i + j] *= c;
}

/***************************************************************************/

void print_disks (REAL *y,
                  WORD  ndisks)
{
    WORD i, j;

    for (i = 0; i < ndisks; i++)
    {
        for (j = 0; j < DIM; j++)
            printf ("%lf ", y[i*DIM + j]);

        printf ("\n");
    }
}

/***************************************************************************/

void conf_init (REAL    *y,
                WORD     ndisks)
{
    WORD  i;
    REAL  delta_x ,delta_y;
    WORD  n;
    WORD  side;

    side = (WORD) sqrt ((REAL) ndisks);

    if (side*side == ndisks)
         n = side;
    else
         n = side + 1;

    delta_x = g_box_size[0] / ((REAL) n);
    delta_y = g_box_size[1] / ((REAL) n);

    for (i = 0; i < ndisks; i++)
    {
        y[2*i+1] = delta_y / 2.0 + (i / n) * delta_y;

        if ((i / n) % 2 == 0)
            y[2*i]   = (i % n) * delta_x + delta_x / 4.0;
        else
            y[2*i]   = delta_x / 2.0 + (i % n) * delta_x + delta_x / 4.0;
    }
}

/***************************************************************************/

void init (REAL   *y,
           WORD    ndisks)
{
    WORD i, j, k;

    conf_init (y, ndisks);

    y[DIM + 0] = y[0] + g_ww_radius + g_width * 2.0; 

    impuls_init (y, ndisks);
}

/***************************************************************************/

void  rescale_momenta (REAL  *y,
                       WORD   ndisks)
{
    WORD   i, j;
    REAL   s, c;
    REAL   V, K;

    V = pot_energy (y);
    K = kin_energy (y);

    c = (g_energy-V) / K;

    if (c < 0.0)
    {
	printf ("RESCALE_MOMENTA> V=%lf > E=%lf\n", V, g_energy);
	return;
    }

    c = sqrt(c);

    for (i = 0; i < ndisks; i++)
        for (j = 0; j < DIM; j++)
            y[DIM*ndisks + DIM*i + j] *= c;
}

/***************************************************************************/

void  print_input_params (INPUT_STR   *input)
{
      time_t  ltime;
      REAL    rho;

      /* density */
      rho  = ((REAL) g_n_disks) / g_box_volume;

      ltime = time (NULL);

      printf ("\ndiatomic in WCA-fluid                           %s", ctime (&ltime));
      printf ("----------------------------------------------------------------\n\n");

      printf ("rho                     %9.7lf\n",   rho);
      printf ("box size(X)             %lf\n",   g_box_size[0]);
      printf ("box size(Y)             %lf\n",   g_box_size[1]);
      printf ("side ratio (x/y)        %lf\n",   2.0/sqrt(3.0));
      printf ("#time steps             %ld\n",   input->nsteps);
      printf ("step size               %lf\n",   input->stepsize);
      printf ("total time              %lf\n",   input->stepsize * input->nsteps);
      printf ("averaging rate          %d\n",    NAVERAGE);
      printf ("# of disks              %d\n",    input->ndisks);
      printf ("disk radius             %lf\n",   1.0);
      printf ("energy                  %lf\n",   g_energy);
      printf ("R_A                     %lf\n",   input->R_A);
      printf ("R_B                     %lf\n",   input->R_B);
      printf ("IM potential height     %lf\n",   input->height);
      printf ("IM potential width      %lf\n",   input->width);
      printf ("IM potential center     %lf\n",   pow(2, 1./6.) + input->width);
      printf ("---------------------------------------------------------------------\n");
}

/***************************************************************************/

void update_avrg (REAL  *y,
                  AVRG_STR *avrg)
{
    WORD i;
    REAL K;
    REAL p[DIM];

    K = kin_energy (y);
    tot_mom (y, p, g_n_disks);

    avrg->ekin    += K;
    avrg->e2kin   += K * K;
    avrg->state_A += h_A (y);
    avrg->state_B += h_B (y);

    for (i = 0; i < DIM; i++)
        avrg->p[i] += p[i];
}

/***************************************************************************/

void init_avrg (AVRG_STR *avrg)
{
    WORD i;

    avrg->ekin     = 0.0;
    avrg->e2kin    = 0.0;
    avrg->state_A  = 0.0;
    avrg->state_B  = 0.0;

    for (i = 0; i < DIM; i++)
        avrg->p[i] = 0.0;
}

/***************************************************************************/

void print_results (DWORD navrg,
                    AVRG_STR  *avrg)
{
    printf ("#average                     %ld\n",   navrg);
    printf ("<Ekin>                       %lf\n",   avrg->ekin / navrg);
    printf ("<px>                         %lf\n",   avrg->p[0] / navrg);
    printf ("<py>                         %lf\n",   avrg->p[1] / navrg);
    printf ("<h_A>                        %lf\n",   avrg->state_A / navrg);
    printf ("<h_B>                        %lf\n",   avrg->state_B / navrg);
}

/*------------------------------------------------------------------------*/

void t_graphics (void)
{
    int win;

    win = OpenX(100, 100);
    OutputTo (win);
    SetLineWidth(2);
    fCircle (50, 50, 10);
    fShow();
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*      MAIN PROGRAM                                                       */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

int   main (int  argc,
            char *argv[])
{
      INPUT_STR  input_str;     
      AVRG_STR   avrg_str;      
      DWORD   i, j;            
      DWORD   count;     
      REAL    x;        
      REAL   *y;        
      REAL   *F;        
      DWORD   navrg;    
      REAL    step;          
      FILE   *fp_bin;        
      DWORD   elaps_time;    
      REAL    cpu_time;      
      char    tstr[20];      
      time_t  ltime;         
      time_t  t_dum;         
      REAL    diff;            
      REAL    p[DIM];          
      DWORD   ncross;
      REAL    old_state;
      REAL    state_HA, state_hA, state_hB; 

      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /*     Parameter Ein- und Ausgabe                                       */
      /*                                                                      */
      /*----------------------------------------------------------------------*/

      /* Parameterpruefung */
      if (argc == 1)
      {
          /* Eingabe der Parameter */
          input ("para.dat", &input_str);
      }
      else
      {
          /* Eingabe der Parameter */
          input (argv[1], &input_str);
      }

      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /*     Initialisierungen                                                */
      /*                                                                      */
      /*----------------------------------------------------------------------*/

      /* initialize random number generator */
      g_idum = time (&t_dum); 
      /* g_idum = 1234567;*/

      /* initialize global variables */
      g_n_disks     = input_str.ndisks;
      g_n_conf      = input_str.ndisks * DIM;
      g_n_phas      = input_str.ndisks * 2 * DIM;
      g_rho         = input_str.rho;
      g_box_size[0] = sqrt (g_n_disks / g_rho * 2.0 / sqrt(3.0));
      g_box_size[1] =  g_box_size[0] * sqrt(3.0) / 2.0;
      g_box_volume  = g_box_size[0] * g_box_size[1];
      g_stepsize    = input_str.stepsize;
      g_ww_radius   = pow(2.0, 1./6.);
      g_ww_radius_2 = g_ww_radius * g_ww_radius;
      g_energy      = input_str.energy;
      g_height      = input_str.height;
      g_width       = input_str.width;
      g_R_A         = input_str.R_A;
      g_R_B         = input_str.R_B;

      for (i = 0; i < DIM; i++)
          g_box_size_2[i] =  g_box_size[i] / 2.0;

      /* Mimimum image condition */
      for (i = 0; i < DIM; i++)
      {
           if (g_box_size[i] < 2.0 * g_ww_radius)
           {
               printf ("MAIN>Minimum image condition is violated!\n");
               printf ("MAIN>Box size = %lf\n", g_box_size[i]);
               printf ("MAIN>Exit!\n");
               exit (0);
           }
      }

      /* allocate arrays */
      y         = ralloc (g_n_phas);
      F         = ralloc (g_n_conf);
      g_y       = ralloc (g_n_phas);
      g_F       = ralloc (g_n_conf);

      /* start timer */
      elaps_time = time (NULL);

      /* output parameters */
      print_input_params (&input_str);

      /* initialize system */
      init (y, input_str.ndisks);

      /* initial state */
      g_state = h_A (y);

#ifdef  GRAPHICS
      /* initialize graphics */
      if (input_str.nplot != 0)
      {
          init_graphics (0.0, 1.0 * g_box_size[0],
                         0.0, 1.0 * g_box_size[1]);
          draw_disks (y, input_str.ndisks);
      }
#endif

      /* initialize averages */
      init_avrg (&avrg_str);

      /* check total momentum */
      tot_mom (y, p, g_n_disks);
      printf ("P_INIT> %lf %lf\n", p[0], p[1]); 

      /* initialize counters */
      navrg = 0;
      ncross = 0;

      /* initialize forces */
      force (y, F);

      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /*    MAINLOOP                                                          */
      /*                                                                      */
      /*----------------------------------------------------------------------*/

      for (x = 0.0, count = 1; count <= input_str.nsteps; x += input_str.stepsize, count++)
      {
          /* propagate for one time step */
          propagate (y, F, input_str.stepsize, input_str.ndisks);

          /* get state */
          g_state = H_A (y);

          /* crossing? */
	  if (g_state != old_state)
	      ncross++;
	  old_state = g_state;

          /* average */
          if (count % NAVERAGE == 0)
          {
              navrg++;
              update_avrg (y, &avrg_str);
              avrg_str.navrg = navrg;
          }


#ifdef GRAPHICS
          if ((input_str.nplot != 0) && ((count % input_str.nplot) == 0))
              draw_disks (y, input_str.ndisks);
#endif

          /* output */
          if ((count % input_str.nio) == 0)
          {
              tot_mom (y, p, g_n_disks);
              printf ("n = %ld  t = %lf  E = %lf  P = %lf %lf   state = %lf  ncross = %ld\n", 
		      count, x + g_stepsize,tot_energy (y), p[0], p[1], g_state, ncross);

              /* Buffer flushen */
              fflush (NULL);
          }
      }

      /*----------------------------------------------------------------------*/
      /*     output results                                                   */
      /*----------------------------------------------------------------------*/

#ifdef GRAPHICS
      /* Close Graphics */
      if (input_str.nplot != 0)
         close_graphics ();
#endif

      /* print results */
      print_results (navrg, &avrg_str);
      printf ("crossings                    %ld\n", ncross);

      /* flush buffer */
      fflush (NULL);

      /* Ende der Zeitmessung */
      elaps_time = time (NULL) - elaps_time;


      printf ("--------------------------------------------------------\n\n");

#ifdef AIX
      printf ("CPU-Time(sec)            %lf\n", cputime ());
      printf ("CPU-Time                 %s\n", str_time (cpu_time = cputime (), tstr));
      printf ("Elapsed_time             %s\n", str_time ((REAL) elaps_time, tstr));
      printf ("CPU-Time/step            %g\n", cpu_time / input_str.nsteps);
#else
      printf ("CPU_time                 %s\n", str_time ((REAL) elaps_time, tstr));
      printf ("CPU-Time/step            %g\n", elaps_time / ((REAL) input_str.nsteps));
#endif
      return (0);
}
