/* ********************************************************************** */
/* * GRAPHICS.C                                                         * */
/* ********************************************************************** */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "odeinty.h"
#include "graphics.h"
#include "diat_md.h"

#include "Xpgp.h"

/* #define POSTSCRIPT */
#define PSFILE        "out.ps"

/***************************************************************************/

extern DWORD         g_collision;
extern REAL          g_rho;
extern REAL          g_energy;
extern REAL          g_stepsize;

extern REAL          g_box_size[DIM];
extern WORD          g_n_disks;
extern WORD          g_n_lya;
extern WORD          g_last_disk_1;
extern WORD          g_last_disk_2;
extern REAL         *g_charge;
extern REAL          g_EE[DIM];
extern REAL          g_state;

DWORD  g_last_coll;
REAL   g_ttime;
REAL  *g_yold;
REAL  *g_lold;

/***************************************************************************/
/*                                                                         */
/*       Globale Daten fuer Graphik                                        */
/*                                                                         */
/***************************************************************************/

#define NCOLOR  5
WORD  g_npart;
REAL  g_xmin, g_xmax;
REAL  g_ymin, g_ymax;
REAL  g_x_length, g_y_length;
WORD  g_radius;
WORD  g_x_zero, g_y_zero;
WORD  g_windowy;
WORD  g_color[NCOLOR];
WORD  g_lyawinx;
WORD  g_lyawiny;
WORD  g_lyad;
int   g_disk_win;
int   g_lya_win;
int   g_erase;


/*------------------------------------------------------------------------*/

void convert_point (REAL  x,
                    REAL  y,
                    WORD  *nx,
                    WORD  *ny)
{
    *nx =             (WORD) ((x - g_xmin) / (g_xmax - g_xmin) * WINDOWX);
    *ny = g_windowy - (WORD) ((y - g_ymin) / (g_ymax - g_ymin) * g_windowy);

#ifdef POSTSCRIPT
    *nx += 100;
    *ny += 150;
#endif

}

/*------------------------------------------------------------------------*/

void test_graphics (void)
{
    int win;

    win = OpenX(100, 100);
    OutputTo (win);
    SetLineWidth(2);
    fCircle (50, 50, 10);
    fShow();
    for (;;);
}

/*------------------------------------------------------------------------*/

void init_graphics (REAL   xmin,
                    REAL   xmax,
                    REAL   ymin,
                    REAL   ymax)
{
    /* Globale Variable versorgen */
    g_xmin = xmin;
    g_xmax = xmax;
    g_ymin = ymin;
    g_ymax = ymax;

    g_x_length = xmax - xmin;
    g_y_length = ymax - ymin;
    g_windowy  = (WORD) ((g_ymax - g_ymin) / (g_xmax - g_xmin) * WINDOWX);

    g_radius = (WORD) (DISK_RADIUS * WINDOWX) / g_x_length;

    /* Buffer allocieren */
    if ((g_yold = (REAL *) malloc (sizeof (REAL) * 4 * g_n_disks)) == NULL)
    {
        printf ("INIT_GRAPHICS> Fehler beim allokieren von %ld Bytes!!\n",
                sizeof (REAL) * 4 * g_n_disks);
        exit (0);
    }

    /* Buffer allocieren */
    if ((g_lold = (REAL *) malloc (sizeof (REAL) * 4 * g_n_disks)) == NULL)
    {
        printf ("INIT_GRAPHICS> Fehler beim allokieren von %ld Bytes!!\n",
                sizeof (REAL) * 4 * g_n_disks);
        exit (0);
    }

    g_lyad = (int) ((REAL) LYAWIN) / ((REAL) (2 * g_n_disks));
    g_lyawinx =  g_lyad * 2 * g_n_disks;
    g_lyawiny = g_lyawinx;

    /* Window oeffnen */
#ifdef POSTSCRIPT
    g_disk_win = OpenPS(PSFILE, PORT);
#else
    g_disk_win = OpenX(WINDOWX, g_windowy);
#endif

    /* Farben */
    g_color[0] = 4;
    g_color[1] = 1;
    g_color[2] = 14;
    g_color[3] = 18;
    g_color[4] = 30;
}

/*------------------------------------------------------------------------*/

void draw_border (void)
{
    WORD nx, ny;
    WORD mx, my;
    WORD i;

    convert_point (g_xmin, g_ymin, &nx, &ny);
    convert_point (g_xmax, g_ymax, &mx, &my);

    fRectangleS(nx, ny, mx, my);
}

/*------------------------------------------------------------------------*/

void draw_arrow (REAL  x,
                 REAL  y,
                 REAL  px,
                 REAL  py)
{
    WORD nx, ny;
    WORD mx, my;
    REAL ax, ay;
    REAL ex, ey;
    REAL rx, ry;

    rx = px / 2.0;
    ry = py / 2.0;

    convert_point (x,    y,    &nx, &ny);
    convert_point (x+rx, y+ry, &mx, &my);

    fLine (nx, ny, mx, my);

    ax = -px / sqrt(px*px + py*py) * ARROW_LENGTH;
    ay = -py / sqrt(px*px + py*py) * ARROW_LENGTH;

    ex = x + rx + cos (ARROW_ANGLE) * ax - sin (ARROW_ANGLE) * ay;
    ey = y + ry + sin (ARROW_ANGLE) * ax + cos (ARROW_ANGLE) * ay;

    convert_point (ex, ey, &nx, &ny);

    fLine (nx, ny, mx, my);

    ex = x + rx + cos (ARROW_ANGLE) * ax + sin (ARROW_ANGLE) * ay;
    ey = y + ry - sin (ARROW_ANGLE) * ax + cos (ARROW_ANGLE) * ay;

    convert_point (ex, ey, &nx, &ny);

    fLine (nx, ny, mx, my);
}

/*------------------------------------------------------------------------*/

void draw_circle (REAL  x,
                  REAL  y,
                  REAL  px,
                  REAL  py,
                  REAL  r,
                  REAL  t,
                  WORD  c)
{
    WORD nx, ny;
    WORD i;
    WORD ncolor, pcolor;

        Pen(c);

        convert_point (x, y, &nx, &ny);
        fCircle (nx, ny, r);

	if (x > g_box_size[0]-1.0)
	{
            convert_point (x-g_box_size[0], y, &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x < 1.0)
	{
            convert_point (x+g_box_size[0], y, &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (y > g_box_size[1]-1.0)
	{
            convert_point (x, y-g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (y < 1.0)
	{
            convert_point (x, y + g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x > g_box_size[0]-1.0 && y > g_box_size[1]-1.0)
	{
            convert_point (x-g_box_size[0], y - g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x > g_box_size[0]-1.0 && y < 1.0)
	{
            convert_point (x-g_box_size[0], y + g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x < 1.0 && y > g_box_size[1] - 1.0)
	{
            convert_point (x+g_box_size[0], y - g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x < 1.0 && y < 1.0)
	{
            convert_point (x+g_box_size[0], y + g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}

        /* draw_arrow (x, y, px, py); */
        /*
        if (ch < 0.0)
           fLine (nx - CROSS/2, ny, nx + CROSS/2, ny);
        else
        {
           fLine (nx - CROSS/2, ny, nx + CROSS/2, ny);
           fLine (nx, ny - CROSS/2, nx, ny + CROSS/2);
        }
        */
}
/*------------------------------------------------------------------------*/

void draw_disks (REAL  *y,
                 WORD   ndisks)
{
     WORD  i, k;
     WORD  nx, ny;
     WORD  mx, my;
     static DWORD count=0;
     static REAL  last_time ;
     WORD   col;

     OutputTo (g_disk_win);
     SetLineWidth(2);

#ifndef POSTSCRIPT
     if (count != 0)
     {
         for (i = 0; i < ndisks; i++)
             draw_circle (g_yold[i*2], g_yold[i*2+1],
                          g_yold[(i+g_n_disks)*2], g_yold[(i+g_n_disks)*2+1],g_radius, last_time, 0);
     }
#endif

     if (g_state == 1.0)
	 col = 31; 
     else
	 col = 4;

     for (i = 0; i < ndisks; i++)
     { 
	 if (i == 0 || i == 1)
         draw_circle (y[i*2], y[i*2 + 1], y[(i+g_n_disks)*2], y[(i+g_n_disks)*2 + 1],g_radius, g_ttime, col);
	 else
         draw_circle (y[i*2], y[i*2 + 1], y[(i+g_n_disks)*2], y[(i+g_n_disks)*2 + 1],g_radius, g_ttime, 1);
     }

     /* Vektor speichern */
     count++;
     last_time = g_ttime;
     memcpy ((void *) g_yold, (void *) y, 4 * g_n_disks * sizeof (REAL));

     fShow();

#ifdef POSTSCRIPT
     draw_border ();
     Clear ();
#endif
}


/*------------------------------------------------------------------------*/


void close_graphics ()
{
}

/*------------------------------------------------------------------------*/
