/* *FH*******************************************************************  */
/* *                                                                    *  */
/* * FILE-NAME     : diat_ps.c                                          *  */
/* * FUNCTION      : path sampling for diatomic in WCA-fluid (2d)       *  */
/* *                                                                    *  */
/* * AUTHOR        : Ch. Dellago                                        *  */
/* * CREATION-DATE : 12-02-1998                                         *  */
/* *                                                                    *  */
/* **********************************************************************  */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <signal.h>

#include "odeinty.h"
#include "odeintc.h"
#include "diat_ps.h"

#include <sys/times.h>
#include <sys/types.h>

#ifdef GRAPHICS
extern void init_graphics  (REAL  xmin, REAL  xmax, REAL  ymin, REAL ymax);
extern void draw_disks     (REAL  *r, REAL o, WORD ndisks, WORD i);
extern void draw_order     (REAL  *o, DWORD nworm);
extern void close_graphics (void);
#endif


/*****************************************************************************/
/*      Global Variables                                                     */
/****************************************************************************/

static long g_idum;                   /* Seed for random number generator    */
REAL        g_time;                   /* time                                */
DWORD       g_nconf;                  /* Dimension of configuration space    */
DWORD       g_ntau;                   /* Number of time-steps                */
DWORD       g_nworm;                  /* Number of configurations            */
DWORD       g_npart;                  /* Number of particles                 */
DWORD       g_nphas;                  /* Dimension of phase space            */
DWORD       g_P_nphas;                /* Dimension of the whole path         */
DWORD       g_P_nconf;                /* Dimension of the configurations     */
REAL        g_beta;                   /* beta = 1 / kT                       */
REAL        g_temp;                   /* kT                                  */
REAL        g_energy;                 /* energy                              */
REAL        g_timestep;             
REAL        g_delta_p;
REAL        g_rho;
REAL        g_dummy;
REAL       *g_order;
REAL       *g_ytry;
REAL       *g_f;
REAL       *g_d;
REAL       *g_fnew;
REAL       *g_fold;
REAL       *g_rs;
REAL       *g_shot_start;
REAL       *g_shot_end;
WORD        g_start;
WORD        g_max_shift;
WORD        g_nsub;
REAL        g_dt_sub;
REAL        g_height;
REAL        g_width;
REAL        g_state;
REAL        g_center;
REAL        g_TST_energy;
REAL        g_TST_n;
REAL        g_2_xx, g_2_yy;
REAL        g_xx, g_yy;

REAL        g_box_size[DIM];              /* Groesse der Box                 */
REAL        g_box_size_2[DIM];            /* Seitenlaenge der Box            */
REAL        g_box_volume;                 /* Flaeche der Box                 */
REAL        g_ww_radius;                  /*                                 */
REAL        g_ww_radius_2;                /*                                 */

DWORD       g_nshot_try, g_nshot_acc;
DWORD       g_nshift_try, g_nshift_acc;

REAL       *g_F;
REAL       *g_F_0;                    /* Forces at t=0                       */
REAL       *g_F_1;                    /* Forces at t=1                       */

REAL       *g_y;                      /*                                     */

REAL       *g_ys; 
REAL       *g_Fs;

REAL        g_R_A, g_R_B, g_R_C;
REAL        g_R_A2, g_R_B2, g_R_C2;

FILE       *g_ergfile;
char        g_infile[50];
WORD        g_start;
char       *g_getfile;
FILE       *g_xmolfile;

REAL       *g_y_from;
REAL       *g_y_to;
REAL       *g_y_shoot;

int         g_my_PE;
int         g_nr_PE;

REAL       *g_nvect[DIM];
REAL       *g_dp;

DWORD      *g_TS_alive;
DWORD      *g_nA;
DWORD      *g_nB;

/***************************************************************************/

double ran3(void)
{
  static int ma[60],mj,mk,mz,i,ii,k,inext,inextp,iff,mbig,mseed;

  if (iff ==0) {
    iff=1;
    mbig = 1000000000;
    /* mseed = 161803398; */
    mseed = g_idum; 
    mz =0;
    mj = mseed ;
    mj = mj % mbig;
    ma[55] = mj;
    mk = 1;
    for (i=1;i<55;i++){
      ii = (21*i) % 55;
      ma[ii] = mk;
      mk = mj - mk;
      if (mk<mz) mk=mk+mbig;
      mj = ma[ii];
    }
    for(k=1;k<=4;k++){
      for(i=1;i<=55;i++){
        ma[i] = ma[i] - ma[1 + ((i+30)%55)];
        if (ma[i]<mz) ma[i] = ma[i] + mbig;
      }
    }
    inext=0;
    inextp=31;
   }
  if (++inext == 56)   inext  =1;
  if (++inextp ==56)  inextp =1;
  mj = ma[inext] - ma[inextp];
  if (mj<mz) mj=mj+mbig;
  ma[inext]=mj;
  return  (double)mj / mbig;
}

/***************************************************************************/

double gssran(void)

{
  double fac,rsq,v1,v2;
  static int iset=0;
  static double gset;

  if (iset==0)  {
    do{
      v1 =2.*ran3() -1.;
      v2 =2.*ran3() -1.;
      rsq = v1*v1 + v2*v2;
    } while ((rsq >= 1)|| (rsq ==0));
    fac = sqrt(-2.*log(rsq)/rsq);
    gset = v1*fac;
    iset =1;
    return v2*fac;
  } else {
    iset =0;
    return gset;
  }
}

/*****************************************************************************/

unsigned int ran (void)
{
    /* Seed im Datensegment */
    static unsigned int next = 667;

    next = next * 1103515245 + 12345;

    return (next);
}

/***************************************************************************/

double fran (long *idum)
{
    long          k;
    float         ans;

    *idum ^= MASK;
    k = (*idum) / IQ;
    *idum = IA * (*idum - k * IQ) - IR * k;
    if (*idum < 0) *idum += IM;
    ans = AM * (*idum);
    *idum ^= MASK;
    return ((double)(ans));
}

/***************************************************************************/

REAL gauss (long *idum)
{
    REAL  v1;
    REAL  v2;
    REAL  r;
    REAL  fac;

    do
    {
        /* v1 = 2.0 * fran (idum) - 1.0;
        v2 = 2.0 * fran (idum) - 1.0;*/

        v1 = 2.0 * ran3 () - 1.0;
        v2 = 2.0 * ran3 () - 1.0;

        r = v1 * v1 + v2 * v2;
    } while (r > 1.0 || r == 0.0);

    fac = sqrt (-2.0 * log (r) / r);

    return (v1 * fac);
}

/***************************************************************************/

REAL cputime (void)
{
    struct tms time_buff;

    times (&time_buff);

    return ((REAL) time_buff.tms_utime / (REAL) CLK_TCK);
}

/***************************************************************************/

REAL timer (WORD  opcode)
{
    static unsigned int stime;

    switch (opcode)
    {
        case START:
        {
            stime = clock ();
            return (0.0);
        }
        case STOP:
            return ((REAL) (clock () - stime) / CLOCKS_PER_SEC);

        default:
            fprintf (g_ergfile,"Parameter error!!!\n");
            return (0.0);
    }
}

/***************************************************************************/

char  next_char (FILE *stream)
{
     static WORD  gl_state = CODE;
     char         prev_c   = EOF;
     int          c;

     while ((c = getc (stream)) != EOF)
     {
          switch (gl_state)
          {
             case CODE:
                 if (c == '/')
                    gl_state = TO_COMM;
                 else
                    return (c);
                 break;

             case TO_COMM:
                 if (c == '*')
                    gl_state = COMM;
                 else
                 {
                    ungetc (c, stream);
                    return (prev_c);
                 }
                 break;

             case COMM:
                 if (c == '*')
                    gl_state = TO_CODE;
                 break;

             case TO_CODE:
                 if     (c == '/')
                    gl_state = CODE;
                 else if (c != '*')
                    gl_state = COMM;
                 break;
          }
          prev_c = c;
     }

     return (EOF);
}

/***************************************************************************/

char *read_line (FILE  *stream,
                 char  *string,
                 WORD   length)
{
     char   *cp;
     int     c;

     cp = string;

     for (c = next_char(stream); length > 0 && c != EOF && c != '\n'; )
     {
           *cp++= c;
            c   = next_char (stream);
     }

     if (length == 0)
     {
         fprintf (g_ergfile,"READ_LINE> Line too long !!\n");
         return (NULL);
     }

     *cp = '\0';

     if (c == EOF)
        return (NULL);
     else
        return (string);
}

/***************************************************************************/

char  *str_time (REAL  seconds,
                 char *tstring)
{

      REAL    sec;                  
      WORD    minutes;              
      WORD    hours;                

      hours      = ((DWORD) seconds) / 3600;
      minutes    = ((DWORD) seconds) / 60 - hours * 60;
      sec        = seconds - hours * 3600 - minutes * 60;

      sprintf (tstring, "%02d:%02d:%05.2lf", hours, minutes, sec);

      return (tstring);
}

/***************************************************************************/

REAL scalprod (REAL  *v1,
               REAL  *v2,
               WORD   dim)
{
      REAL   result;

      for (result = 0.0; dim > 0; dim--, v1++, v2++)
           result += (*v1) * (*v2);

      return (result);
}

/***************************************************************************/

void  input (char   *filename,
             INPUT_STR  *input_str)
{
      FILE  *stream;
      char   line_buffer[256];
      int    nl;

      if ((stream = fopen (filename, "r")) == NULL)
      {
           fprintf (g_ergfile,"INPUT> No file %s\n", filename);
           exit (0);
      }

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->ncycles));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->nrelax));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->ntau));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->CPU_secs));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->nio));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->navrgout));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->ngraph));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->nTS));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->start));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->npart));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%ld", &(input_str->max_shift));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->temp));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->rho));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->timestep));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%d", &(input_str->substeps));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->R_A));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->R_B));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->R_C));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->delta_p));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->shots));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->height));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%lf", &(input_str->width));

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%s", input_str->ergfile);

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%s", input_str->filename);

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%s", input_str->savefile);

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%s", input_str->getfile);

      read_line (stream, line_buffer, 256);
      sscanf (line_buffer, "%s", input_str->xmolfile);

      fclose (stream);
}

/***************************************************************************/

DWORD *dalloc (DWORD  size)
{
      DWORD  *address;

      if ((address = (DWORD *) malloc (sizeof (DWORD) * size)) == NULL)
      {
          fprintf (g_ergfile,"Fehler beim allokieren von %ld Bytes!!\n", sizeof (DWORD) * size);
          exit (0);
      }

      return (address);
}

/***************************************************************************/

REAL *ralloc (DWORD  size)
{
      REAL  *address;

      if ((address = (REAL *) malloc (sizeof (REAL) * size)) == NULL)
      {
          fprintf (g_ergfile,"Fehler beim allokieren von %ld Bytes!!\n", sizeof (REAL) * size);
          exit (0);
      }

      return (address);
}

/***************************************************************************/

REAL **palloc (DWORD  size)
{
      REAL  **address;

      if ((address = (REAL **) malloc (sizeof (REAL *) * size)) == NULL)
      {
          fprintf (g_ergfile,"Fehler beim allokieren von %ld Bytes!!\n", sizeof (REAL *) * size);
          exit (0);
      }

      return (address);
}

/******************************************************************************/

void  print_input_params (INPUT_STR   *input)
{
      time_t  ltime;

      /* Zeit und Datum feststellen */
      ltime = time (NULL);

      fprintf (g_ergfile,"\nDIATOMIC IN LJ-FLUID             %s", ctime (&ltime));
      fprintf (g_ergfile,"----------------------------------------------------------------\n");
#ifdef END_IN_B
      fprintf (g_ergfile,"Path ensemble                         start in A end in B\n");
#else
      fprintf (g_ergfile,"Path ensemble                         start in A visit B\n");
#endif

#ifdef CANONICAL
      fprintf (g_ergfile,"Ensemble                              CANONICAL\n");
#else
      fprintf (g_ergfile,"Ensemble                              MICROCANONICAL\n");
#endif

      fprintf (g_ergfile,"Integrator                            VERLET\n");
      fprintf (g_ergfile,"Pair potential                        WCA\n");
      fprintf (g_ergfile,"Dimension                             %d\n",  DIM);
      fprintf (g_ergfile,"Number of particles                   %ld\n", input->npart);
      fprintf (g_ergfile,"Number of time steps                  %ld\n", input->ntau);
      fprintf (g_ergfile,"Number of system copies               %ld\n", input->ntau + 1);
      fprintf (g_ergfile,"Number of cycles                      %ld\n", input->ncycles);
      fprintf (g_ergfile,"Number of relaxation steps            %ld\n", input->nrelax);
      fprintf (g_ergfile,"Output rate                           %ld\n", input->nio);
      fprintf (g_ergfile,"Output averages                       %ld\n", input->navrgout);
      fprintf (g_ergfile,"Configuration space dimension         %ld\n", g_nconf);
      fprintf (g_ergfile,"Phase space dimension                 %ld\n", g_nphas);
      fprintf (g_ergfile,"Maximum shift                         %ld\n", input->max_shift);
#ifdef CANONICAL
      fprintf (g_ergfile,"beta                                  %lf\n", 1. / input->temp);
      fprintf (g_ergfile,"kT                                    %lf\n", input->temp);
#else
      fprintf (g_ergfile,"Energy                                %lf\n", input->temp);
#endif
      fprintf (g_ergfile,"time step                             %lf\n", input->timestep);
      fprintf (g_ergfile,"number of substeps                    %d\n", input->substeps);
      fprintf (g_ergfile,"substep                               %lf\n", g_dt_sub);
      fprintf (g_ergfile,"Averaging rate                        %d\n",  NAVERAGE);
      fprintf (g_ergfile,"Drawing rate                          %ld\n", input->ngraph);
      fprintf (g_ergfile,"PS-file                               %s\n",  input->filename);
      fprintf (g_ergfile,"Constraint                            distance from COM\n");
      fprintf (g_ergfile,"R_A                                   %lf\n", g_R_A);
      fprintf (g_ergfile,"R_B                                   %lf\n", g_R_B);
      fprintf (g_ergfile,"R_C                                   %lf\n", g_R_C);
      fprintf (g_ergfile,"Region A                              r < R_A\n");
      fprintf (g_ergfile,"Region B                              r > R_B\n");
      fprintf (g_ergfile,"delta p                               %lf\n", input->delta_p);
      fprintf (g_ergfile,"fraction of shots                     %lf\n", input->shots);
      fprintf (g_ergfile,"fraction of shifts                    %lf\n", 1.0 - input->shots);
      fprintf (g_ergfile,"IM potential height                   %lf\n", input->height);
      fprintf (g_ergfile,"IM potential width                    %lf\n", input->width);
      fprintf (g_ergfile,"IM potential center                   %lf\n", pow(2, 1./6.) + input->width);
      fprintf (g_ergfile,"boxsize-x                             %lf\n", g_box_size[0]);
      fprintf (g_ergfile,"boxsize-y                             %lf\n", g_box_size[1]);
      fprintf (g_ergfile,"rho                                   %lf\n", g_rho);
#ifdef TSE         
      fprintf (g_ergfile,"With TS-ensemble                      \n");
      fprintf (g_ergfile,"Number of shots                       %d\n", NSHOTS);
      fprintf (g_ergfile,"Maximum number of steps per shot      %d\n", SHOT_STEPS);
#endif

      if (input->start == COLD)
      {
           fprintf (g_ergfile,"Start                                 cold\n");
      }
      else
      {
           fprintf (g_ergfile,"Start                                 warm\n");
           fprintf (g_ergfile,"Read path from                        %s\n",  input->getfile);
      }
      fprintf (g_ergfile,"Save path to                          %s\n",  input->savefile);
#ifdef XMOL
      fprintf (g_ergfile,"Xmol path to                          %s\n",  input->xmolfile);
#endif

#ifdef CRAY_T3E
      fprintf (g_ergfile,"My PE                                 %d\n",  g_my_PE);
      fprintf (g_ergfile,"Number of PEs                         %d\n",  g_nr_PE);
#endif
      fprintf (g_ergfile,"----------------------------------------------------------------\n");
}

/***************************************************************************/

void  remap (REAL *y,
             WORD  ndisks)
{
    WORD i, j;

    for (i = 0; i < ndisks; i++)
    {
        for (j = 0; j < DIM; j++)
        {
            if (y[i*DIM + j] >  g_box_size[j]) y[i*DIM + j] -= g_box_size[j];
            if (y[i*DIM + j] <  0.0)           y[i*DIM + j] += g_box_size[j];
        }
    }
}

/***************************************************************************/

void  q_image (REAL  *y1,
               REAL  *y2,
               REAL  *dq)
{
    WORD  i;
    REAL  xx, yy;

    dq[0] = y2[0] - y1[0];
    if (dq[0] >  g_box_size_2[0])
        dq[0] -= g_box_size[0];
    if (dq[0] < -g_box_size_2[0])
        dq[0] += g_box_size[0];

    dq[1] = y2[1] - y1[1];
    if (dq[1] >  g_box_size_2[1])
        dq[1] -= g_box_size[1];
    if (dq[1] < -g_box_size_2[1])
        dq[1] += g_box_size[1];
}

/***************************************************************************/

REAL distance (REAL  *y)
{
    REAL  q[DIM];
    WORD  i; 
    REAL  r;

    q_image (y+0*DIM, y+1*DIM, q);

    for (i = 0, r = 0.0; i < DIM; i++)
        r += q[i] * q[i];
    
    return (sqrt(r));
}

/***************************************************************************/

REAL pot_energy (REAL  *y)
{
    REAL  s;
    WORD  i, j, k;
    REAL  q[DIM];
    REAL  r;
    REAL r6;
    REAL  z, a, w;
    REAL  xx, yy;
    WORD  iD, jD;

    a = g_ww_radius;
    w = g_width;

    s = 0.0;

    for (i = 0; i < g_npart - 1; i++)
    {
        for (j = i + 1; j < g_npart; j++)
        {
            iD = i * DIM;
            jD = j * DIM;

            xx = y[jD]   - y[iD];
            yy = y[jD+1] - y[iD+1];

            if (xx >  g_2_xx) xx -= g_xx;
            if (xx < -g_2_xx) xx += g_xx;

            if (yy >  g_2_yy) yy -= g_yy;
            if (yy < -g_2_yy) yy += g_yy;

            r = xx * xx + yy * yy;

            if (r < g_ww_radius_2)
            {
                r6 = 1.0 / (r * r * r);

                s += (4.0 * r6) * (r6 - 1.0) + 1.0;
            }

            if (i == 0 && j == 1)
            {
                z  = sqrt(r) - w - a;
                s += g_height * (1-z*z/(w*w)) * (1-z*z/(w*w));
            }
        }
    }

    return (s);
}

/***************************************************************************/

REAL diatomic_energy (REAL  *y)
{
    REAL  s;
    WORD  i, j, k;
    REAL  q[DIM];
    REAL  r, r6;
    REAL  z, a, w;
    REAL  xx, yy;

    a = g_ww_radius;
    w = g_width;

    s = 0.0;

    xx = y[DIM]   - y[0];
    yy = y[DIM+1] - y[1];

    if (xx >  g_2_xx) xx -= g_xx;
    if (xx < -g_2_xx) xx += g_xx;

    if (yy >  g_2_yy) yy -= g_yy;
    if (yy < -g_2_yy) yy += g_yy;

     r = xx * xx + yy * yy;

     if (r < g_ww_radius_2)
     {
         r6 = 1.0 / (r * r * r);

         s += (4.0 * r6) * (r6 - 1.0) + 1.0;
     }

     z  = sqrt(r) - w - a;
     s += g_height * (1-z*z/(w*w)) * (1-z*z/(w*w));

     return (s);
}

/***************************************************************************/

REAL kin_energy (REAL *y)
{
    WORD i;
    REAL K;

    for (i = 0, K = 0.0; i < g_nconf; i++)
        K += y[g_nconf + i] * y[g_nconf + i];

    return (K / 2.0);
}

/***************************************************************************/

void tot_mom (REAL  *y,
              REAL  *p,
              WORD   ndisks)
{
    REAL  s;
    WORD  i, j;

    for (j = 0; j < DIM; j++)
        p[j] = 0.0;

    for (i = 0; i < ndisks; i++)
        for (j = 0; j < DIM; j++)
            p[j] += y[DIM * ndisks + i * DIM + j];
}

/***************************************************************************/

REAL tot_energy (REAL *y)
{
    return (pot_energy (y) + kin_energy (y));
}

/***************************************************************************/

void force (REAL *y, 
	    REAL *F)
{
    WORD i, j, k;
    WORD ip, jp;
    WORD iD, jD;
    REAL r6, r, r2, r2i, f, d;
    REAL q[DIM];
    REAL  z, a, w;
    REAL  xx, yy;

    a = g_ww_radius;
    w = g_width;

    /* Initialize forces */
    for (i = 0; i < g_nconf; i++)
        F[i]=0.0;

    /* Interaction between particles */
    for (i = 0; i < g_npart - 1; i++)
    {
        for (j = i + 1; j < g_npart; j++)
        {
	    iD = i * DIM;
	    jD = j * DIM;

            xx = y[jD]   - y[iD];
            yy = y[jD+1] - y[iD+1];

            if (xx >  g_2_xx) xx -= g_xx;
            if (xx < -g_2_xx) xx += g_xx;

            if (yy >  g_2_yy) yy -= g_yy;
            if (yy < -g_2_yy) yy += g_yy;

            r2 = xx * xx + yy * yy;

            if (r2 < g_ww_radius_2)
            {
                r6 = r2 * r2 * r2;

                f = 24.0 / (r6 * r2) * (2.0 / r6 - 1.0);

                F[iD]   -= f * xx;
                F[iD+1] -= f * yy;

                F[jD]   += f * xx;
                F[jD+1] += f * yy;
            }

            if (i == 0 && j == 1)
            {
                r = sqrt(r2);
                z = r - w - a;

                f = 4.0 * g_height * (1.-z*z/(w*w)) * z / (w*w) / r;

                F[0] -= f * xx;
                F[1] -= f * yy;

                F[2] += f * xx;
                F[3] += f * yy;
            }
        }
    }
}

/***************************************************************************/

REAL solvent_force (REAL *y)
{
    WORD i, j, k;
    WORD ip, jp;
    WORD iD, jD;
    REAL r6, r, r2, r2i, f, d;
    REAL q[DIM];
    REAL  z, a, w;
    REAL  xx, yy;
    REAL  F[DIM];

    a = g_ww_radius;
    w = g_width;

    q[0] = y[DIM + 0] - y[0];
    q[1] = y[DIM + 1] - y[1];

    r = sqrt(q[0] * q[0] + q[1] * q[1]);

    q[0] /= r;
    q[1] /= r;

    F[0] = 0.0;
    F[1] = 0.0;

    /* Interaction between particles */
    for (i = 2; i < g_npart; i++)
    {
	for (j = 0; j < 2; j++)
	{
	    iD = i * DIM;
	    jD = j * DIM;

            xx = y[jD]   - y[iD];
            yy = y[jD+1] - y[iD+1];

            if (xx >  g_2_xx) xx -= g_xx;
            if (xx < -g_2_xx) xx += g_xx;

            if (yy >  g_2_yy) yy -= g_yy;
            if (yy < -g_2_yy) yy += g_yy;

            r2 = xx * xx + yy * yy;

            if (r2 < g_ww_radius_2)
            {
                r6 = r2 * r2 * r2;

                f = 24.0 / (r6 * r2) * (2.0 / r6 - 1.0);

                if (j == 1)
		{
                    F[0] -= f * xx;
                    F[1] -= f * yy;
		}
		else
		{
                    F[0] += f * xx;
                    F[1] += f * yy;
		}
            }
	}
    }

    return (F[0] * q[0] + F[1] * q[1]);
}

/***************************************************************************/

void print_results (char      *string,
		    AVRG_STR  *avrg,
		    DWORD      step)
{
    WORD  i;
    REAL  E, E2;
    REAL  K, K2;
    REAL  S, S2;
    REAL  r;
    REAL  hB, hBdot;
    REAL  dF_dR;
    REAL  r_shot, r_shift;
    REAL  nA, nB;
    REAL  acc;
    DWORD att;
    REAL  F;

    nA = ((REAL) avrg->nA) / avrg->navrg;
    nB = ((REAL) avrg->nB) / avrg->navrg;

    if (g_nshot_try > 0)
        r_shot = ((REAL) g_nshot_acc) / ((REAL) g_nshot_try);

    if (g_nshift_try > 0)
        r_shift = ((REAL) g_nshift_acc) / ((REAL) g_nshift_try);
 
    fprintf (g_ergfile,"Shooting steps tried               %ld\n", g_nshot_try);
    fprintf (g_ergfile,"Shooting steps accepted            %ld\n", g_nshot_acc);
    if (g_nshot_try != 0)
    fprintf (g_ergfile,"Shooting acceptance rate           %lf\n", r_shot);
    fprintf (g_ergfile,"Shifting steps tried               %ld\n", g_nshift_try);
    fprintf (g_ergfile,"Shifting steps accepted            %ld\n", g_nshift_acc);
    if (g_nshift_try != 0)
    fprintf (g_ergfile,"Shifting acceptance rate           %lf\n", r_shift);

    fprintf (g_ergfile,"Beads in A                            %lf\n", nA);
    fprintf (g_ergfile,"Beads in B                            %lf\n", nB);
    fprintf (g_ergfile,"Fraction of beads in A                %lf\n", nA/(g_ntau + 1));
    fprintf (g_ergfile,"Fraction of beads in B                %lf\n", nB/(g_ntau + 1));
    fprintf (g_ergfile,"dH between first and last time slice  %lf\n", avrg->dE / avrg->navrg);
    fprintf (g_ergfile,".................................................................\n");
    fprintf (g_ergfile,"      n     t      <V>       <K>      K+V     hB    hBdot    r      F     sh _acc \n");

    for (i = 0; i < g_nworm; i++)
    {
        E     = avrg->epot[i]   / avrg->navrg;
        K     = avrg->ekin[i]   / avrg->navrg;
        E2    = avrg->epot2[i]  / avrg->navrg;
        K2    = avrg->ekin2[i]  / avrg->navrg;
	hB    = avrg->h_B[i]    / avrg->navrg;
	hBdot = avrg->hdot_B[i] / avrg->navrg;
	r     = avrg->r[i]   / avrg->navrg;
	F     = avrg->F[i]   / avrg->navrg;

        att = avrg->shot_acc[i] + avrg->shot_rej[i];

	if (att == 0)
	    acc =-1.0;
	else
	    acc = ((REAL) avrg->shot_acc[i]) / att;

        if ((E2 - E * E)< 0.0)
            E2 = 0.0;
        else
            E2 = E2-E*E;


        if ((K2 - K * K) < 0.0)
            K2 = 0.0;
        else
            K2 = K2-K*K;

        fprintf (g_ergfile,"%s %2d  %lf %lf %lf %lf %lf %lf %lf %lf %lf\n",
                 string, i, i * g_timestep, E, K, K + E, hB, hBdot, r, F, acc);
    }
    fprintf (g_ergfile,"................................................................\n");
}

/***************************************************************************/

REAL metro (REAL x)
{
    if (x < 1.0)
        return (x);
    else
        return (1.0);
}

/***************************************************************************/

void shot (REAL *xold,
           REAL *xnew,
	   DWORD nsteps)
{
    DWORD i, j;
    REAL  dt;

    /* substep */
    dt = g_dt_sub;

    memcpy (g_ys, xold, sizeof(REAL) * g_nphas);
    force  (g_ys, g_Fs);

    for (i = 0; i < g_nsub * nsteps; i++)
    {
        /* propagate coordinates */
        for (j = 0; j < g_nconf; j++)
	{
             xnew[j] = g_ys[j] + dt * g_ys[g_nconf + j] + 
                       dt * dt * g_Fs[j] * 0.5;
    
             xnew[g_nconf + j] = g_ys[g_nconf + j] +
                                 dt * g_Fs[j] * 0.5;
	}

        /* Calculate new forces */
        force (xnew, g_Fs);

        /* propagate momenta */
        for (j = 0; j < g_nconf; j++)
            xnew[g_nconf + j] += dt * g_Fs[j] * 0.5;
	
        memcpy (g_ys, xnew, sizeof(REAL) * g_nphas);
    }
}

		
/***************************************************************************/

void go_forward (REAL *xold,
                 REAL *xnew,
		 REAL *Fold,
		 REAL *Fnew)
{
    DWORD i, j;
    REAL  dt, dt2, dtt2;
    REAL *vnew;

    /* substep */
    dt   = g_dt_sub;
    dt2  = dt * 0.5;
    dtt2 = dt * dt * 0.5;
    vnew = xnew + g_nconf;

    memcpy (xnew, xold, sizeof(REAL) * g_nphas);
    memcpy (Fnew, Fold, sizeof(REAL) * g_nconf);

    for (i = 0; i < g_nsub; i++)
    {
        /* propagate coordinates */
        for (j = 0; j < g_nconf; j++)
	{
            xnew[j] += dt  * vnew[j] + dtt2 * Fnew[j];
            vnew[j] += dt2 * Fnew[j];
	}
    
        /* Calculate new forces */
        force (xnew, Fnew);

        /* propagate momenta */
        for (j = 0; j < g_nconf; j++)
            vnew[j] += dt2 * Fnew[j];
    }
}
		
/***************************************************************************/

void go_backward (REAL *xold,
                  REAL *xnew,
		  REAL *Fold,
		  REAL *Fnew)
{
    DWORD i, j;
    REAL  dt, dt2, dtt2;
    REAL *vnew;

    /* substep */
    dt = g_dt_sub;
    dt2  = dt * 0.5;
    dtt2 = dt * dt * 0.5;
    vnew = xnew + g_nconf;

    memcpy (xnew, xold, sizeof(REAL) * g_nphas);
    memcpy (Fnew, Fold, sizeof(REAL) * g_nconf);

    for (i = 0; i < g_nsub; i++)
    {
        /* propagate coordinates */
        for (j = 0; j < g_nconf; j++)
	{
            xnew[j] -= dt  * vnew[j] - dtt2 * Fnew[j];
            vnew[j] -= dt2 * Fnew[j];
	}

        /* Calculate new forces */
        force (xnew, Fnew);

        /* propagate momenta */
        for (j = 0; j < g_nconf; j++)
            vnew[j] -= dt2 * Fnew[j];
    }
}

/***************************************************************************/

void propagate (REAL  *y,
                REAL  *F,
	        DWORD start,
		DWORD forward,
		DWORD backward)
{
    DWORD i, j;
    REAL  *xold, *xnew;
    REAL  *Fold, *Fnew;

    /* propagate in forward direction */
    for (i = 0; i < forward; i++)
    {
	xold = y + (start + i + 0) * g_nphas;
	xnew = y + (start + i + 1) * g_nphas;

	Fold = F + (start + i + 0) * g_nconf;
	Fnew = F + (start + i + 1) * g_nconf;

        go_forward (xold, xnew, Fold, Fnew);

        remap (xnew, g_npart);
    }

    /* propagate in backward direction */
    for (i = 0; i < backward; i++)
    {
	xold = y + (start - i - 0) * g_nphas;
	xnew = y + (start - i - 1) * g_nphas;

	Fold = F + (start - i - 0) * g_nconf;
	Fnew = F + (start - i - 1) * g_nconf;

        go_backward (xold, xnew, Fold, Fnew);

        remap (xnew, g_npart);
    }
}

/***************************************************************************/

void shift_path (REAL  *ynew,
		 REAL  *yold,
		 REAL  *Fnew,
		 REAL  *Fold)
{
    WORD  shift;
    DWORD nshift;

    shift  = (WORD) (ran3() * g_max_shift) + 1;
    nshift = g_nworm - shift;

    if (ran3() < 0.5)
    {
	/* right shift */
	memcpy (ynew + shift * g_nphas, yold, sizeof(REAL) * nshift * g_nphas);
	memcpy (Fnew + shift * g_nconf, Fold, sizeof(REAL) * nshift * g_nconf);

	/* complete path */
	propagate (ynew, Fnew, shift, 0, shift);
    }
    else
    {
	/* left shift */
	memcpy (ynew, yold + shift * g_nphas, sizeof(REAL) * nshift * g_nphas);
	memcpy (Fnew, Fold + shift * g_nconf, sizeof(REAL) * nshift * g_nconf);

	/* complete path */
	propagate (ynew, Fnew, g_ntau - shift, shift, 0);
    }
}

/***************************************************************************/

void invert_path (REAL  *ynew,
	  	  REAL  *yold,
		  REAL  *Fnew,
		  REAL  *Fold)
{
    WORD i, j;

    /* invert path and forces */
    for (i = 0; i <= g_ntau; i++)
    {
	memcpy (ynew + (g_ntau - i) * g_nphas, yold + i * g_nphas, sizeof(REAL) * g_nphas);
	memcpy (Fnew + (g_ntau - i) * g_nconf, Fold + i * g_nconf, sizeof(REAL) * g_nconf);
    }

    /* invert the momenta of the new path */
    for (i = 0; i <= g_ntau; i++)
	for (j = 0; j < g_nconf; j++)
	    ynew[i * g_nphas + g_nconf + j] *= -1.0;
}

/***************************************************************************/

REAL H_A (REAL  *y)
{
    REAL  q[DIM];
    REAL  r, z;

    q_image (y+0*DIM, y+1*DIM, q);

    r = sqrt(q[0]*q[0] + q[1]*q[1]);

    z = r - g_ww_radius - g_width;

    if (z < 0.0)
        return (1.0);
    else
        return (0.0);
}

/***************************************************************************/

REAL h_A (REAL  *y)
{
    REAL  q[DIM];
    REAL  r, z;

    q_image (y+0*DIM, y+1*DIM, q);

    r = q[0]*q[0] + q[1]*q[1];

    if (r < g_R_A2)
        return (1.0);
    else
        return (0.0);
}

/***************************************************************************/


REAL h_B (REAL  *y)
{
    REAL  q[DIM];
    REAL  r;

    q_image (y+0*DIM, y+1*DIM, q);

    r = q[0]*q[0] + q[1]*q[1];

    if (r > g_R_B2)
        return (1.0);
    else
        return (0.0);
}

/***************************************************************************/

REAL h_C (REAL *y)
{
    REAL  q[DIM];
    REAL  r;

    q_image (y+0*DIM, y+1*DIM, q);

    r = q[0]*q[0] + q[1]*q[1];

    if (r > g_R_C2)
        return (1.0);
    else
        return (0.0);
}

/***************************************************************************/

REAL h_P_B (REAL *y)
{
    WORD i;

#ifdef END_IN_B
    if (h_B (y + g_ntau * g_nphas) == 1.0 && h_C (y + g_ntau * g_nphas) == 0.0)
        return (1.0);
    else
	return (0.0);
#else
    if (h_C (y + g_ntau * g_nphas) == 1.0)
	return (0.0);

    for (i = 0; i <= g_ntau; i++)
       if (h_B (y + (g_ntau - i) * g_nphas) == 1.0)
           return (1.0);

    return (0.0);
#endif
}

/***************************************************************************/

REAL hdot_B (REAL *yold,
	     REAL *ynew)
{
    return ((h_B(ynew) - h_B(yold)) / g_timestep);
}

/***************************************************************************/

REAL transition_time (REAL *y)
{
    WORD i, iA;
    REAL hA, hAold;

    hA    = 1.0; 
    hAold = 1.0;

    for (i = 0; i <= g_ntau; i++)
    {
       hA = h_A (y + i * g_nphas);

       if (hA == 0.0 && hAold == 1.0)
	   iA = i;

       if (h_B (y + i * g_nphas) == 1.0)
           return ((i - iA + 1) * g_timestep);
	
        hAold = hA;
    }

    return (g_ntau * g_timestep);
}

/***************************************************************************/

REAL T_time (REAL *y)
{
    WORD  i, iA;
    REAL  hA, hAold;
    REAL  rA;
    REAL  q[DIM];
    REAL *x;
    REAL  r, z;
    REAL  rnew, rold;
    REAL  r1, r2;
    REAL  tA, tB;

    hA    = 1.0; 
    hAold = 1.0;

    rold = 0.0;

    for (i = 0; i <= g_ntau; i++)
    {
	x = y + i * g_nphas;
	q_image (x, x+DIM, q);

	rnew = q[0]*q[0] + q[1]*q[1];

        if (rnew >= g_R_A2 && rold < g_R_A2)
	{
	    r1 = sqrt(rold);
	    r2 = sqrt(rnew);
	    tA = ((i-1) + (g_R_A - r1) / (r2 - r1)) * g_timestep;
        }

        if (rnew > g_R_B2)
	{
	    r1 = sqrt(rold);
	    r2 = sqrt(rnew);
	    tB = ((i-1) + (g_R_B - r1) / (r2 - r1)) * g_timestep;

	    return (tB - tA);
	}
	
	rold = rnew;
    }

    return (g_ntau * g_timestep);
}

/***************************************************************************/

WORD head (REAL *y)
{
    WORD n_head;

    for (n_head = g_ntau; h_A(y + n_head * g_nphas) == 0.0; n_head--);
	if (n_head == 0.0)
	    return (n_head);

    return (n_head);
}

/***************************************************************************/

WORD tail (REAL *y)
{
    WORD n_tail;

    for (n_tail = 0; n_tail <g_ntau && h_B(y + n_tail * g_nphas) == 0.0; n_tail++);

    return (n_tail);
}


/***************************************************************************/

WORD count_nA (REAL *y)
{
    WORD i, n_A;

    for (i = 0, n_A = 0; i < g_nworm; i++)
	if (h_A(y + i * g_nphas) == 1.0)
	    n_A++;

    return (n_A);
}

/***************************************************************************/

WORD count_nB (REAL *y)
{
    WORD i, n_B;

    for (i = 0, n_B = 0; i < g_nworm; i++)
	if (h_B(y + i * g_nphas) == 1.0)
	    n_B++;

    return (n_B);
}

/***************************************************************************/

void init_nvect (void)
{
    WORD i, j;
    REAL c;

    /* normal vectors */
    for (i = 0; i < DIM; i++)
        g_nvect[i] = ralloc (g_nconf);

    /* normalization factor */
    c = 1.0 / sqrt((REAL) g_npart); 

    /* initialize */
    for (i = 0; i < DIM; i++)
	for (j = 0; j < g_nconf; j++)
	    g_nvect[i][j] = 0.0;

    /* initialize momentum vectors */
    for (i = 0; i < g_npart; i++)
        for (j = 0; j < DIM; j++)
	    g_nvect[j][i * DIM + j] = c; 
}

/***************************************************************************/

void adjust_PK (REAL *y)
{
    WORD  i, j, k;
    REAL  s, K, V, c;

    /* kill normal components */
    for (i = 0; i < DIM; i++)
    {
	s = scalprod (y + g_nconf, g_nvect[i], g_nconf);

	for (k = 0; k < g_nconf; k++)
	    y[g_nconf + k] -= s * g_nvect[i][k];
    } 

    /* Adjust kinetic energy */
    K = kin_energy (y);
    V = pot_energy (y);

    printf ("V %lf  K %lf  E %lf\n",  V, K, g_energy);

    c = sqrt((g_energy - V) / K);

    for (i = 0; i < g_nconf; i++)
	y[g_nconf + i] *= c;
}

/***************************************************************************/

void check_PK (REAL *y)
{
    REAL L;
    REAL P[DIM];
    REAL K, V, E;

    tot_mom (y, P, g_npart);

    K = kin_energy (y);
    V = pot_energy (y);

    fprintf (g_ergfile, "CHECK_PK> P %lf %lf\n", P[0], P[1]);
    fprintf (g_ergfile, "CHECK_PK> K %lf  V %lf  E %lf\n", K, V, K + V);
}

/***************************************************************************/

void update_average (REAL *y,
                     REAL *F,
                     AVRG_STR *avrg)
{
    DWORD i;
    REAL  E;
    REAL  K;
    REAL  p;

    avrg->navrg++;

    avrg->nA += count_nA (y);
    avrg->nB += count_nB (y);

    avrg->dE += tot_energy (y + g_ntau * g_nphas) - tot_energy (y);

    /* Energies and fluctuations */
    for (i = 0; i < g_nworm; i++)
    {
        /* potential energy */
        E = pot_energy (y + i * g_nphas);

        avrg->epot[i]   += E;
        avrg->epot2[i]  += E * E;

        /* kinetic energy */
        E = kin_energy (y + i * g_nphas);

        avrg->ekin[i]   += E;
        avrg->ekin2[i]  += E * E;

	avrg->h_B[i] += h_B (y + i * g_nphas);
	avrg->r[i] += distance (y + i * g_nphas);
	avrg->F[i] += solvent_force (y + i * g_nphas);
    }

    /* hB_dot */
    for (i = 0; i < g_nworm-1; i++)
	avrg->hdot_B[i] += hdot_B(y + i * g_nphas, y + (i+1) * g_nphas);
}

/***************************************************************************/

void random_momenta (REAL *y, 
                     REAL  Ksoll)
{
    WORD  i;
    REAL  sx, sy;
    REAL  K, V;
    REAL  c;

    sx = 0.0;
    sy = 0.0;

    for (i = 0; i < g_npart; i++)
    {
       y[g_nconf + i * DIM + 0] = gauss (&g_idum) * sqrt(g_temp);
       y[g_nconf + i * DIM + 1] = gauss (&g_idum) * sqrt(g_temp);

       sx += y[g_nconf + i * DIM + 0];
       sy += y[g_nconf + i * DIM + 1];
    }

    for (i = 0, K = 0.0; i < g_npart; i++)
    {
       y[g_nconf + i * DIM + 0] -= sx / g_npart;
       y[g_nconf + i * DIM + 1] -= sy / g_npart;

       K += y[g_nconf + i * DIM + 0] * y[g_nconf + i * DIM + 0] + 
            y[g_nconf + i * DIM + 1] * y[g_nconf + i * DIM + 1]; 
    }

    c = sqrt (Ksoll / K);

    for (i = 0; i < g_nconf; i++)
        y[g_nconf + i] *= c;
}

/***************************************************************************/

void prob_AB (REAL *y,
              REAL  K,
              REAL *pA,
              REAL *pB)
{
    DWORD   i, j;
    DWORD   nA, nB;

    /* random shots */
    for (i = 0, nA = 0, nB = 0; i < NSHOTS; i++)
    {
        memcpy ((void *) g_y_shoot, (void *) y, g_nphas * sizeof (REAL));

        /* new_momenta */
        random_momenta (g_y_shoot, K);

        /* shoot */
        shot (g_y_shoot, g_y_shoot, SHOT_STEPS);

        if (h_A (g_y_shoot) == 1.0)
           nA++;

        if (h_B (g_y_shoot) == 1.0)
           nB++;
    }

    *pA = ((REAL) nA) / NSHOTS;
    *pB = ((REAL) nB) / NSHOTS;
}

/***************************************************************************/

void find_TS (REAL *y ,
              REAL *y_TS,
	      WORD *t_TS,
	      WORD *n_TS)
{
    DWORD i, j;
    DWORD ntot, n;
    REAL  xAmax, xAmin, xtot2;
    REAL  xA, xB;
    REAL  pA0, pA1;  
    REAL  K;
    DWORD nalive;

    /* Initialize */
    for (i = 0; i < g_nworm; i++)
    {
	g_nA[i] = 0;
	g_nB[i] = 0;

        if (h_A(y + i * g_nphas) == 0.0 && h_B(y + i * g_nphas) == 0.0)
	    g_TS_alive[i] = 1;
	else
	    g_TS_alive[i] = 0;
    }

    /* to maximum number of shots */
    for (i = 0; i < NSHOTS_MAX; i++)
    {
	for (j = 0; j < g_nworm; j++)
	{
	    /* if point still alive */
	    if (g_TS_alive[j] == 1)
	    {
	        /* copy configuration */
                memcpy (g_y_shoot, y + j * g_nphas, g_nphas * sizeof (REAL));

                K = g_energy - pot_energy (g_y_shoot);

                /* new_momenta */
                random_momenta (g_y_shoot, K);

                /* shoot off trajectory */
                shot (g_y_shoot, g_y_shoot, SHOT_STEPS);

                /* check region */
                if (h_A (g_y_shoot) == 1.0)
                    g_nA[j]++;
 
                if (h_B (g_y_shoot) == 1.0)
                   g_nB[j]++;

                /* check for TS after minimum number of shots */
	        if (i > NSHOTS_MIN)
	        {
		    ntot = g_nA[j] + g_nB[j];
		    xtot2 = ((REAL) ntot) / 2.0;
		    xAmax = g_nA[j] + TS_TOLERANCE * sqrt((REAL) g_nA[j]);
		    xAmin = g_nA[j] - TS_TOLERANCE * sqrt((REAL) g_nA[j]);

	  	    if (ntot != 0 && (xAmax < xtot2 || xAmin > xtot2))
		        g_TS_alive[j] = 0;
	        }
	    }
	}
    }

    /* Get transition states */
    for (i = 0, n = 0; i < g_nworm; i++)
    {
	if (g_TS_alive[i] == 1)
	{
	    memcpy (y_TS + n * g_nconf, y + i * g_nphas, g_nconf * sizeof (REAL));
	    t_TS[n] = i;
	    n++;
	}

        if (i < g_nworm - 1 && (g_nA[i] + g_nB[i]) > 0) 
	{
	    if (g_TS_alive[i] == 0 && g_TS_alive[i+1] == 0)
	    {
	        pA0 = ((REAL) g_nA[i]) /   (g_nA[i]   + g_nB[i]);
	        pA1 = ((REAL) g_nA[i+1]) / (g_nA[i+1] + g_nB[i+1]);

		if ((pA0 - 0.5) * (pA1 - 0.5) < 0.0)
		{
		    if (fabs (pA0 - 0.5) < fabs (pA1 - 0.5))
		    {
	                memcpy (y_TS + n * g_nconf, y + i * g_nphas, g_nconf * sizeof (REAL));
	                t_TS[n] = i;
		    }
		    else
		    {
	                memcpy (y_TS + n * g_nconf, y + (i+1) * g_nphas, g_nconf * sizeof (REAL));
	                t_TS[n] = i + 1;
		    }

	            n++;
		}
            }
	}

	if (n > N_TS_MAX)
	    break;
    }

    *n_TS = n;
}

/***************************************************************************/

void print_profile (char  *string,
		    REAL  *y,
		    REAL  *F)
{
    DWORD i;
    REAL  E, K, p;
    REAL  V0, K0;
    REAL  hB;
    REAL  pA, pB, pAold;
    REAL  r, rold;
    REAL  gF;
    REAL  qF;

    fprintf (g_ergfile,".................................................................\n");
#ifdef TSE
    fprintf (g_ergfile,"       t       V        K      K+V     h_B      r     qF    pA     pB     pA+pB\n");
#else
    fprintf (g_ergfile,"       t       V        K      K+V     h_B      r     qF\n");
#endif

    pAold = 1.0;
    rold  = 0.0;
    
    /* Energies and fluctuations */
    for (i = 0; i < g_nworm; i++)
    {
        /* potential energy */
        E = pot_energy (y + i * g_nphas);
        K = kin_energy (y + i * g_nphas);
	hB = h_B (y + i * g_nphas);
	r  = distance (y + i * g_nphas);
	qF = solvent_force (y + i * g_nphas);

#ifdef TSE
        if (i > 50)
        prob_AB (y + i * g_nphas, K, &pA, &pB);
	else
	{
	    pA = 0.0;
	    pB = 0.0;
	}

        fprintf (g_ergfile,"%s %ld %lf %lf %lf %lf %lf %lf %lf %lf %lf\n", 
		 string, i, E, K, E+K, hB, r, qF, pA, pB, pA + pB);
#else
        fprintf (g_ergfile,"%s %ld %lf %lf %lf %lf %lf %lf\n", 
		 string, i, E, K, E+K, hB, r, qF);
#endif
    }
    fprintf (g_ergfile,"----------------------------------------------------------------\n");
}

/***************************************************************************/

void init_average (AVRG_STR   *avrg)
{
    DWORD i;

    avrg->navrg = 0;

    avrg->epot   = ralloc (g_nworm);
    avrg->epot2  = ralloc (g_nworm);
    avrg->ekin   = ralloc (g_nworm);
    avrg->ekin2  = ralloc (g_nworm);
    avrg->h_B    = ralloc (g_nworm);
    avrg->hdot_B = ralloc (g_nworm);
    avrg->r      = ralloc (g_nworm);
    avrg->F      = ralloc (g_nworm);

    avrg->shot_rej = dalloc (g_nworm);
    avrg->shot_acc = dalloc (g_nworm);

    avrg->nA = 0;
    avrg->nB = 0;

    avrg->dE = 0.0;

    for (i = 0; i < g_nworm; i++)
    {
        avrg->epot[i]   = 0.0;
        avrg->epot2[i]  = 0.0;
        avrg->ekin[i]   = 0.0;
        avrg->ekin2[i]  = 0.0;
        avrg->h_B[i]    = 0.0;
        avrg->hdot_B[i] = 0.0;
        avrg->r[i]      = 0.0;
        avrg->F[i]      = 0.0;

	avrg->shot_acc[i] = 0;
	avrg->shot_rej[i] = 0;
    }
}

/***************************************************************************/

void update_shot (AVRG_STR *avrg,
                  WORD      n,
		  WORD      acc)
{
	if (acc == ACCEPTED)
	    avrg->shot_acc[n]++;
	else
	    avrg->shot_rej[n]++;
}

/***************************************************************************/

void init_forces (REAL  *y,
                  REAL  *F)
{
   DWORD  i, j;

   for (i = 0; i < g_nworm; i++)
       force (y + i * g_nphas,  F + i * g_nconf);
}


/***************************************************************************/

void mom_init (REAL *y)
{
    WORD  i;

    for (i = 0; i < g_nconf; i++)
        y[g_nconf + i] = gauss (&g_idum);
}

/***************************************************************************/

void conf_init (REAL    *y,
                WORD     ndisks)
{
    WORD  i;
    REAL  delta_x ,delta_y;
    WORD  n;
    WORD  side;

    side = (WORD) sqrt ((REAL) ndisks);

    if (side*side == ndisks)
         n = side;
    else
         n = side + 1;

    delta_x = g_box_size[0] / ((REAL) n);
    delta_y = g_box_size[1] / ((REAL) n);

    for (i = 0; i < ndisks; i++)
    {
        y[2*i+1] = delta_y / 2.0 + (i / n) * delta_y;

        if ((i / n) % 2 == 0)
            y[2*i]   = (i % n) * delta_x + delta_x / 4.0;
        else
            y[2*i]   = delta_x / 2.0 + (i % n) * delta_x + delta_x / 4.0;
    }
}

/***************************************************************************/

void get_path (REAL   *y,
	       char   *filename)
{
    DWORD  dim;
    DWORD  npart;
    DWORD  nworm;
    DWORD  i, j;
    FILE  *infile;

    if ((infile = fopen (filename, "rb")) == NULL)
    {
        fprintf (g_ergfile,"GET_PATH> Fehler beim Oeffnen der Datei %s\n", filename);
        exit (0);
    }

    /* Read parameters */
    fread (&dim,   sizeof (DWORD), 1,infile);
    fread (&npart, sizeof (DWORD), 1,infile);
    fread (&nworm, sizeof (DWORD), 1,infile);

    /* Check parameters */
    if (dim != DIM || npart != g_npart || nworm != g_nworm)
    {
        fprintf (g_ergfile,"GET_PATH> Parameter are inkonsistent\n");
        fprintf (g_ergfile,"GET_PATH> Dimension  %ld  %ld\n", (DWORD) DIM, dim);
        fprintf (g_ergfile,"GET_PATH> npart      %ld  %ld\n",  g_npart, npart);
        fprintf (g_ergfile,"GET_PATH> nworm      %ld  %ld\n",  g_nworm, nworm);
        exit (0);
    }

    /* Read Path */
    fread (y, sizeof (REAL), g_nphas * g_nworm, infile);

    /* Close file */
    fclose (infile);
}

/***************************************************************************/

void save_path (REAL   *y,
	        char   *filename)
{
    DWORD  i, j;
    DWORD  dim, npart, nworm;
    FILE  *outfile;

    if ((outfile = fopen (filename, "wb")) == NULL)
    {
        fprintf (g_ergfile,"SAVE_PATH> Fehler beim Oeffnen der Datei %s\n", filename);
        exit (0);
    }

    /* write parameters */
    dim   = DIM;
    npart = g_npart;
    nworm = g_nworm;

    fwrite (&dim,   sizeof (DWORD), 1, outfile);
    fwrite (&npart, sizeof (DWORD), 1, outfile);
    fwrite (&nworm, sizeof (DWORD), 1, outfile);

    /* write path */
    fwrite (y, sizeof (REAL), g_nphas * g_nworm, outfile);

    /* Close file */
    fclose (outfile);
}

/***************************************************************************/

void ascii_path (REAL   *y,
	         char   *filename)
{
    DWORD  i, j;
    DWORD  dim, npart, nworm;
    FILE  *outfile;

    if ((outfile = fopen (filename, "w")) == NULL)
    {
        fprintf (g_ergfile,"SAVE_PATH> Fehler beim Oeffnen der Datei %s\n", filename);
        exit (0);
    }

    for (j = 0; j < g_nworm; j++)
    {
        for (i = 0; i < g_npart; i++)
        {
            fprintf (outfile, "%12.8lf %12.8lf %12.8lf %12.8lf\n",
                     y[j * g_nphas + i * DIM + 0],
                     y[j * g_nphas + i * DIM + 1],
		     y[j * g_nphas + g_nconf + i * DIM + 0],
		     y[j * g_nphas + g_nconf + i * DIM + 1]);
        }
    }

    fclose (outfile);

    fflush (NULL);
}

/***************************************************************************/

void xmol_path (REAL *y)
{
    WORD i, j;
    REAL hA, hB;
    char c;
    
    for (j = 0; j < g_nworm; j++)
    {
        fprintf (g_xmolfile, "%d\n\n", g_npart);

        for (i = 0; i < g_npart; i++)
        {
	    if (i == 0 || i == 1)
	    {
		hA = h_A (y + j * g_nphas);
		hB = h_B (y + j * g_nphas);

		if (hA == 1.0)
		    c = 'O';
		else if (hB == 1.0)
		    c = 'N';
		else
		    c = 'C';
	    }
	    else 
		c = 'H';

            fprintf (g_xmolfile, "%c %lf %lf %lf\n", c,
                     y[j * g_nphas + i * DIM + 0],
                     y[j * g_nphas + i * DIM + 1], 0.0);
        }
    }

    fflush (NULL);
}

/***************************************************************************/

#ifdef GRAPHICS
void draw_path (REAL *y)
{
    WORD i, j;
    REAL hA, hB;
    REAL r;
    char c;

    for (j = 0; j < g_nworm; j++)
    {
        hA = h_A (y + j * g_nphas);
        hB = h_B (y + j * g_nphas);
        r  = distance (y + j * g_nphas);

        for (i = 0, g_dummy = 0.0; i < 50000; i++)
            g_dummy += ran3();

        if (hA == 1.0)
            g_state = STATE_A;
        else if (hB == 1.0)
            g_state = STATE_B;
        else
            g_state = STATE_0;

        draw_disks (y + j * g_nphas, r, g_npart, j);
    }
}

/***************************************************************************/

void draw_profile (REAL *y)
{
    WORD i, j;
    char c;

    for (j = 0; j < g_nworm; j++)
        g_order[j] = distance (y + j * g_nphas);

    draw_order (g_order, g_nworm);
}
#endif

/***************************************************************************/

void shift_A (REAL  *y,
	      REAL  *F)
{
    DWORD i, j;

    /* look for a bead in A */
    for (i = 0; i < g_nworm; i++)
	if (h_A (y + i * g_nphas) == 1.0)
	    break;

    if (i == g_nworm)  
    {
        fprintf (g_ergfile, "SHIFT_A> No point in A\n");
	return;
    }

    if (i == g_ntau)
	return;
    
    fprintf (g_ergfile, "SHIFT_A> Initial path shifted by            %ld\n", i);

    for (j = 0; j <= g_ntau - i; j++)
    {
	memcpy (y + j * g_nphas, y + (i + j) * g_nphas, sizeof(REAL) * g_nphas);
	memcpy (F + j * g_nconf, F + (i + j) * g_nconf, sizeof(REAL) * g_nconf);
    }

    /* complete path */
    propagate (y, F, g_ntau - i, i, 0);
}

/***************************************************************************/

void shift_B (REAL  *y,
	      REAL  *F)
{
    int  i, j;

    /* look for a bead in B */
    for (i = 0; i < g_nworm; i++)
	if (h_B (y + i * g_nphas) == 1.0 && h_C (y + i * g_nphas) == 0.0)
	    break;

    if (i == g_nworm)  
    {
        fprintf (g_ergfile, "SHIFT_B> No point in B && C\n");
	return;
    }

    if (i == g_ntau)
	return;

    fprintf (g_ergfile, "SHIFT_B> Initial path shifted by            %ld\n", g_ntau - i);
    
    for (j = i; j >= 0; j--)
    {
	memcpy (y + (j + g_ntau - i) * g_nphas, y + j * g_nphas, sizeof(REAL) * g_nphas);
	memcpy (F + (j + g_ntau - i) * g_nconf, F + j * g_nconf, sizeof(REAL) * g_nconf);
    }

    /* complete path */
    propagate (y, F, g_ntau - i, 0, g_ntau - i);
}

/***************************************************************************/

REAL TST_energy (REAL  *y)
{
    DWORD i;
    REAL  r, r1, r2;
    REAL  V, V1, V2;

    if (g_center > g_R_A && g_center < g_R_B)
	r = g_center;
    else
	r = g_R_B;

    V = 0.0;
    r1 = distance (y);

    for (i = 1; i < g_nworm; i++)
    {
	r2 = distance (y + i * g_nphas);

	if ((r2 - r) * (r1 - r) < 0.0)
	{
	    
	    V1 = pot_energy (y + (i - 1) * g_nphas);
	    V2 = pot_energy (y + i * g_nphas);

            if (fabs(r1-r2) < 1.e-10)
	        V = V1;
            else
	        V = V1 + (V2 - V1) * (r1 - r) / (r1 - r2);

            g_TST_energy += V;
	    g_TST_n++;
	}

	r1 = r2;
    }

    return (V);
}

/***************************************************************************/

void init_path (REAL  *y,
                REAL  *F,
		char  *infile)
{
    DWORD  i;
    DWORD  start; 
    REAL   rA, rB;
    REAL   dist;

    start = g_ntau / 2; 

    /* Cold start */
    if (g_start == COLD)
    {
        /* Configuration of first time slice */
        conf_init (y + start * g_nphas, g_npart);

        /* initial distance */
	if (g_R_B < g_R_A)
	{
	    if (g_R_B < g_ww_radius)
		dist = g_ww_radius;
	    else
		dist = g_R_B + 0.001; 
	}
	else
	    dist = g_ww_radius + g_width;

        /* diatomic inital position */
        y[start * g_nphas + DIM] = y[start * g_nphas] + dist;

	/* Momenta of first time slice */
	mom_init (y + start * g_nphas);

	/* radial momentum of diatomic */
        y[start * g_nphas + g_nconf + 0 * DIM + 0] = -1.0;
        y[start * g_nphas + g_nconf + 0 * DIM + 1] =  0.0;

        y[start * g_nphas + g_nconf + 1 * DIM + 0] =  1.0;
        y[start * g_nphas + g_nconf + 1 * DIM + 1] =  0.0;

	/* Adjust PK */
	adjust_PK (y + start * g_nphas);
        check_PK  (y + start * g_nphas);

        /* Initial forces */
        force (y + start * g_nphas, F + start * g_nconf);

	/* Shoot off new path */
	propagate (y, F, start, g_ntau - start, start);

	/* if x_0 not in A try shifting the path */
	if (h_A (y) == 0)
	{
	    rA = distance (y);
	    rB = distance (y + g_ntau * g_nphas);

	    fprintf (g_ergfile, "INIT_PATH> rA %lf    rB %lf\n", rA, rB);

            shift_A (y, F);
	}

	/* final constraint not obeyed */
	if (h_P_B (y) == 0)
	{
	    rA = distance (y);
	    rB = distance (y + g_ntau * g_nphas);

	    fprintf (g_ergfile, "INIT_PATH> rA %lf    rB %lf\n", rA, rB);

            shift_B (y, F);
	}
    }
    else
    {
	/* Read path */
	get_path (y, infile);

        /* initialize forces */
	init_forces (y, F);
    }

    /* Check initial and final constraint */
    if (h_A (y) == 0.0 || h_P_B (y) == 0.0)
    {
	rA = distance (y);
	rB = distance (y + g_ntau * g_nphas);

	fprintf (g_ergfile, "INIT_PATH> rA %lf    rB %lf\n", rA, rB);
	fprintf (g_ergfile, "INIT_PATH> h_A(x_0) %lf\n", h_A (y));
	fprintf (g_ergfile, "INIT_PATH> h_B(x_L) %lf\n", h_B (y + g_ntau * g_nphas));
	fprintf (g_ergfile, "INIT_PATH> h_C(x_L) %lf\n", h_C (y + g_ntau * g_nphas));
        fprintf (g_ergfile, "INIT_PATH> Path is not connecting A and B!\n");
        /* exit (0);*/
    }

#ifdef XMOL
    xmol_path (y);
#endif
}

/***************************************************************************/

void new_momenta (REAL    *y)
{
    WORD  i, j, k;
    REAL  sx, sy, sz;
    REAL  dpx, dpy, dpz;
    REAL  K, V;
    REAL  s;
    REAL  c;
    REAL  a;
    REAL  qcm[DIM];
    REAL  xx, yy;

    /* position of the center of mass of particles 0 and 1 */
    xx = y[2] - y[0];
    yy = y[3] - y[1];

    if (xx >  g_2_xx) xx -= g_xx;
    if (xx < -g_2_xx) xx += g_xx;
    if (yy >  g_2_yy) yy -= g_yy;
    if (yy < -g_2_yy) yy += g_yy;

    qcm[0] = y[0] + xx / 2.0;
    qcm[1] = y[1] + yy / 2.0;

    a = diatomic_energy (y) / g_energy + 0.2;

    a = 1.0;                                  

    /* for all particles */
    for (j = 0; j < g_npart; j++)
    {
	if (j == 0 || j == 1)
	{
            g_dp[j * DIM + 0] = a * g_delta_p * gauss (&g_idum); 
            g_dp[j * DIM + 1] = a * g_delta_p * gauss (&g_idum); 
	}
	else
	{
            g_dp[j * DIM + 0] = a * g_delta_p * gauss (&g_idum); 
            g_dp[j * DIM + 1] = a * g_delta_p * gauss (&g_idum); 
	}
    }

    /* kill normal components */
    for (i = 0; i < DIM; i++) 
    {
	s = scalprod (g_dp, g_nvect[i], g_nconf);

	for (k = 0; k < g_nconf; k++)
	    g_dp[k] -= s * g_nvect[i][k];
    } 

    /* add to old momentum */
    for (k = 0; k < g_nconf; k++)
        y[g_nconf + k] += g_dp[k];

#ifndef CANONICAL
    K = kin_energy (y);
    V = pot_energy (y);

    c = sqrt ((g_energy - V) / K);

    for (i = 0; i < g_nconf; i++)
	y[g_nconf + i] *= c; 
#endif

}

/***************************************************************************/

void path_copy (REAL *ynew,
		REAL *yold,
		REAL *Fnew, 
		REAL *Fold)
{
    memcpy ((void *) ynew, (void *) yold, g_P_nphas * sizeof (REAL));
    memcpy ((void *) Fnew, (void *) Fold, g_P_nconf * sizeof (REAL));
}

/***************************************************************************/

void state_copy (REAL *ynew,
		 REAL *yold,
		 REAL *Fnew, 
		 REAL *Fold)
{
    memcpy ((void *) ynew, (void *) yold, g_nphas * sizeof (REAL));
    memcpy ((void *) Fnew, (void *) Fold, g_nconf * sizeof (REAL));
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*       MAIN PROGRAM                                                      */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

int   main (int  argc,
            char *argv[])
{
      /* Lokale Variable */
      DWORD      i, j;               /* counters                              */
      INPUT_STR  input_str;          /* input structure                       */
      AVRG_STR   avrg_str;           /* averages                              */
      REAL      *y, *ynew;           /* Pointer to path                       */
      REAL      *F, *Fnew;           /* Pointer to path                       */
      DWORD      elaps_time;         /* Elapsed time                          */
      REAL       cpu_time;           /* CPU-Time                              */
      char       tstr[20];           /* Time string                           */
      time_t     t_dum;              /* Dummy fuer seed                       */
      REAL       K, S, s;            /* Generalized energies                  */
      DWORD      niter;              /* number of iterations                  */
      REAL       Einit;              /* Initial Path energy                   */
      FILE      *fp;                 /* Pointer to signal-file                */
      WORD       start;              /* Starting point of shooting            */
      REAL       R, Enew, Eold;      /* Energies                              */
      REAL       dp0[DIM], dp1[DIM]; /* Total momentum                        */
      DWORD      nrelax;
      DWORD      round_trips;
      DWORD      ninvert;
      DWORD      start_up;
      DWORD      nstart_up;
      REAL       rmin, rmax;
      REAL       hmin, hmax;
      REAL       T_trans;
      DWORD      n_trans;
      REAL       E_TST;
      REAL      *y_TS;
      WORD       n_TS;
      DWORD      N_TS, n_search;
      WORD       t_TS[N_TS_MAX];
      WORD       n_rej;

      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /*     parameter input and output                                       */
      /*                                                                      */
      /*----------------------------------------------------------------------*/

      if (argc == 1)
          strcpy (g_infile, "para.dat");
      else
          strcpy (g_infile, argv[1]);

      /* read parameters */
      input (g_infile, &input_str);

      /* Check boundaries */
      if (input_str.R_C <= input_str.R_B)
      {
	    printf ("MAIN> R_C=%lf > R_B=%lf\n", input_str.R_C, input_str.R_B);
	    printf ("MAIN> exit\n");
            exit (0);
      }

      /* Check maxshift */
      if (input_str.max_shift > input_str.ntau)
      {
	    printf ("MAIN> max_shift = %ld > tau = %ld\n", input_str.max_shift, input_str.ntau);
	    printf ("MAIN> exit\n");
            exit (0);
      }

      /* open output file */
      if (strcmp (input_str.ergfile, "stdout") == 0)
          g_ergfile = stdout;
      else
          g_ergfile = fopen (input_str.ergfile, "a");

      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /*     initialize                                                       */
      /*                                                                      */
      /*----------------------------------------------------------------------*/

      /* initialize random number generator */
      /* g_idum = time (&t_dum); */     /* random        */
      g_idum = 12345678;        /* deterministic */

      /* initialize global variables */
      g_npart        = input_str.npart;
      g_nconf        = g_npart * DIM;
      g_nphas        = 2 * g_nconf;
      g_ntau         = input_str.ntau;
      g_nworm        = input_str.ntau + 1;
      g_beta         = 1.0 / input_str.temp;
      g_temp         = input_str.temp;
      g_timestep     = input_str.timestep;
      g_P_nconf      = g_nworm * g_nconf;
      g_P_nphas      = g_nworm * g_nphas;
      g_start        = input_str.start;
      g_delta_p      = input_str.delta_p;
      g_max_shift    = input_str.max_shift;
      g_nsub         = input_str.substeps;
      g_dt_sub       = g_timestep /g_nsub;
      g_height       = input_str.height;
      g_width        = input_str.width;
      g_rho          = input_str.rho;

      /* box geometry */
      g_box_size[0]  = sqrt (g_npart / g_rho * 2.0 / sqrt(3.0));
      g_box_size[1]  = g_box_size[0] * sqrt(3.0) / 2.0;
      g_box_volume   = g_box_size[0] * g_box_size[1];
      g_ww_radius  = pow(2.0, 1./6.);
      g_ww_radius_2= g_ww_radius * g_ww_radius;

      g_center = g_ww_radius + g_width;

      for (i = 0; i < DIM; i++)
          g_box_size_2[i] =  g_box_size[i] / 2.0;

      g_2_xx = g_box_size_2[0];
      g_2_yy = g_box_size_2[1];
      g_xx = g_box_size[0];
      g_yy = g_box_size[1];

#ifdef XMOL
      g_xmolfile = fopen (input_str.xmolfile, "w");
#endif

      if (g_start == COLD)
	  nrelax = input_str.nrelax;
      else
	  nrelax = 0;

#ifdef CANONICAL 
      g_temp = input_str.temp;
#else
      g_energy = input_str.temp;
#endif

      /* Geometry */
      g_R_A       = input_str.R_A;
      g_R_B       = input_str.R_B;
      g_R_C       = input_str.R_C;
      g_R_A2      = g_R_A * g_R_A;
      g_R_B2      = g_R_B * g_R_B;
      g_R_C2      = g_R_C * g_R_C;

      /* allocate arrays */
      y         = ralloc (g_P_nphas);
      ynew      = ralloc (g_P_nphas);
      y_TS      = ralloc (N_TS_MAX * g_nconf);
      F         = ralloc (g_P_nconf);
      Fnew      = ralloc (g_P_nconf);
      g_ytry    = ralloc (g_P_nphas);
      g_F       = ralloc (g_nconf);
      g_f       = ralloc (g_nconf);
      g_d       = ralloc (g_nphas);
      g_fnew    = ralloc (g_nconf);
      g_fold    = ralloc (g_nconf);
      g_rs      = ralloc (g_nphas);
      g_y_shoot = ralloc (g_nphas);

      g_TS_alive = dalloc (g_nworm);
      g_nA       = dalloc (g_nworm);
      g_nB       = dalloc (g_nworm);
      g_order    = ralloc (g_nworm);

      g_shot_start = ralloc (g_nphas);
      g_shot_end   = ralloc (g_nphas);

      g_ys      = ralloc (g_nphas);
      g_Fs      = ralloc (g_nconf);

      g_y_from  = ralloc (g_nphas);
      g_y_to    = ralloc (g_nphas);

      /* Arrays for forces */
      g_F_0     = ralloc (g_nconf);
      g_F_1     = ralloc (g_nconf);

      g_dp = ralloc (g_nconf);

      /* Empty Save-file */
      /* fp = fopen (input_str.savefile, "w");
      fclose (fp);*/

      /* start timer */
      elaps_time = time (NULL);

      /* print parameters */
      print_input_params (&input_str);

      /* initialize normal vectors */
      init_nvect ();

      /* initialize averaging */
      init_average (&avrg_str);

      /* initial path */
      init_path (y, F, input_str.getfile);

      /* initializations */
      g_time         = 0.0;
      Eold = tot_energy (y);
      round_trips = 0;
      start_up =  1;
      ninvert = 0;
      nstart_up = 0;
      rmin = g_R_B + 0.0000001;
      rmax = g_R_C - 0.0000001;
      g_TST_energy = 0.0;
      g_TST_n      = 0;
      T_trans      = 0.0;
      n_trans      = 0;
      N_TS         = 0;
      n_search     = 0;
      n_rej        = 0;

      /* initialize counter */
      g_nshot_try = 0;  
      g_nshot_acc = 0;

      g_nshift_try = 0;  
      g_nshift_acc = 0;

#ifdef  GRAPHICS
      /* Graphik initialisieren */
      if (input_str.ngraph != 0)
          init_graphics (0.0, 1.0 * g_box_size[0],
                         0.0, 1.0 * g_box_size[1]);
#endif


      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /*    MAINLOOP                                                          */
      /*                                                                      */
      /*----------------------------------------------------------------------*/

      /* Loop over all passes */
      for (i = 0; i < input_str.ncycles && cputime () < input_str.CPU_secs; i++)
      {
          if (start_up == 1)
	  {
              if (h_A (y) == 1.0 && h_P_B(y) == 1.0)
	      {
                  fprintf (g_ergfile,"MAIN> %ld. path accepted\n", nstart_up + 1);
		  start_up = 0;
	      }
	      else
		  nstart_up++;
	  }

          if (ran3() < input_str.shots)
	  {
              /*------------------------------------------------------------------*/
              /*     PATH SHOOTING                                                */
              /*------------------------------------------------------------------*/

              g_nshot_try++;

              /* determine starting point */
	      start = (WORD) (ran3() * g_nworm);

	      if (start == g_nworm)
	          start--;

              /* new path */
	      state_copy (ynew + start * g_nphas, y + start * g_nphas,
			  Fnew + start * g_nconf, F + start * g_nconf);

              /* new momenta */
	      new_momenta (ynew + start * g_nphas); 

#ifdef CANONICAL
              /*----------------------------------------------*/
	      /* Canonical distribution                       */
              /*----------------------------------------------*/
	      /* New initial energy */
	      Enew = tot_energy (ynew + start * g_nphas);
	      Eold = tot_energy (y);

	      /* Acceptance factor */
	      R = exp (-g_beta * (Enew - Eold));

              /* Accept or reject */
              if (ran3() < metro (R))
              {
	          /* Shoot off new path */
	          propagate (ynew, Fnew, start, g_ntau - start, start);
  
                  if (h_A (ynew) == 1.0 && h_P_B(ynew) == 1.0)
                  {
	              path_copy (y, ynew, F, Fnew);
                      g_nshot_acc++;
                      update_shot (&avrg_str, start, ACCEPTED);
	          }
	          else
                      update_shot (&avrg_str, start, REJECTED);
              }
              else
                  update_shot (&avrg_str, start, REJECTED);
#else
              /*----------------------------------------------*/
	      /* Microcanonical distribution                  */
              /*----------------------------------------------*/
	      /* Shoot off new path backward */
	      propagate (ynew, Fnew, start, 0, start);
  
              if (h_A (ynew) == 1.0)
              {
	          /* Shoot off new path forward */
	          propagate (ynew, Fnew, start, g_ntau - start, 0);
  
                  if (h_P_B(ynew) == 1.0)
                  {
	              path_copy (y, ynew, F, Fnew);
                      g_nshot_acc++;
                      update_shot (&avrg_str, start, ACCEPTED);
		  }
		  else
		  {
                      update_shot (&avrg_str, start, REJECTED);
		      n_rej++;
		  }
	      }
	      else
	      {
                  update_shot (&avrg_str, start, REJECTED);
		  n_rej++;
	      }
#endif
	  }
	  else
	  {
              /*------------------------------------------------------------------*/
              /*     PATH SHIFTING                                                */
              /*------------------------------------------------------------------*/
              g_nshift_try++;

              /* New path */
	      shift_path (ynew, y, Fnew, F);

              /* Accept or reject */
              if (h_A (ynew) == 1.0 && h_P_B(ynew) == 1.0)
              {
	          path_copy (y, ynew, F, Fnew);
                  g_nshift_acc++;
	      }
          }

          /*------------------------------------------------------------------*/
          /*     PATH INVERSION                                               */
          /*------------------------------------------------------------------*/
	  if (h_A (y + g_ntau * g_nphas) == 1.0)
	  {
	      /* count round trips */
	      round_trips++;

	      /* invert path woth certain probability */
              if (ran3() < 0.1)
	      {
                  /* count path inversions */
		  ninvert++;

                  /* invert path */
	          invert_path (ynew, y, Fnew, F);

                  /* copy to old path */
	          path_copy (y, ynew, F, Fnew);
	      }
	  }

          /*------------------------------------------------------------------*/
          /*     COLLECT DATA                                                 */
          /*------------------------------------------------------------------*/
          /* Update averages */
          if ((i+1) % NAVERAGE == 0 && i > nrelax)
	  {
              update_average (y, F, &avrg_str);
	  }

	  T_trans += T_time (y);
	  n_trans++;

	  /* round trip? */
	  if (h_A (y + g_ntau * g_nphas) == 1.0)
	      round_trips++;

          /* Total momentum */
	  tot_mom (y, dp0, g_npart);
	  tot_mom (y + g_ntau * g_nphas, dp1, g_npart);

          /* Output */
          if ((i+1) % input_str.nio == 0 && i > nrelax)
          {
	      if (g_TST_n > 0)
		  E_TST = g_TST_energy / g_TST_n;
	      else
		  E_TST = 0.0;

	      if (input_str.shots != 1.0)
                  fprintf (g_ergfile,"i %8ld dE %9.6lf K0 %6.3lf V0 %7.3lf E0 %7.3lf Etst %7.3lf %7.3lf shot %5.3lf shift %5.3lf ", 
		           (i+1), 
		           tot_energy (y + g_ntau * g_nphas) - tot_energy (y), 
		           kin_energy (y),
		           pot_energy (y),
		           tot_energy (y),
			   TST_energy (y),
			   E_TST,
		           ((REAL) g_nshot_acc) / ((REAL) g_nshot_try), 
		           ((REAL) g_nshift_acc) / ((REAL) g_nshift_try));
	      else
                  fprintf (g_ergfile,"i %8ld dE %9.6lf K0 %6.3lf V0 %7.3lf E0 %7.3lf Etst %7.3lf %7.3lf shot %5.3lf ", 
		           (i+1), 
		           tot_energy (y + g_ntau * g_nphas) - tot_energy (y), 
		           kin_energy (y),
		           pot_energy (y),
		           tot_energy (y),
			   TST_energy (y),
			   E_TST,
		           ((REAL) g_nshot_acc) / ((REAL) g_nshot_try)); 
			    
	      fprintf (g_ergfile, "\n");
			 

              /* Buffer flushen */
              fflush (NULL);
          }
	  
          /* printout averages and save path */
	  if ((i+1) % input_str.navrgout == 0 && i > nrelax)
	  {
              /* print_results ("AV", &avrg_str, (i+1));*/
              print_profile ("PROF", y, F);
	      ascii_path (y, "ascii_path.dat");
	      save_path (y, input_str.savefile); 
              fflush (NULL);
	  }

          /* Find transition states */           
	  if ((i+1) % input_str.nTS == 0 && i > nrelax)
	  {
	      n_search++;
	      find_TS (y, y_TS, t_TS, &n_TS);
	      printf ("i %ld nTS %ld (t,r) ", i, n_TS);
	      for (j = 0; j < n_TS; j++)
		  printf (" %ld %lf ", t_TS[j], distance (y_TS + j * g_nconf));
	      printf ("\n");

	      N_TS += n_TS;

              fflush (NULL);
	  }

          /* output for xmol or graphics*/
	  if ((i+1) % input_str.ngraph == 0)
	  {
#ifdef GRAPHICS
              draw_profile (y);
              draw_path (y);
#endif
#ifdef XMOL
	      xmol_path (y);
              fflush (NULL);
#endif
	  }
      }

      /*----------------------------------------------------------------------*/
      /*                                                                      */
      /*     Results                                                          */
      /*                                                                      */
      /*----------------------------------------------------------------------*/

      /* Save final path */
      save_path (y, input_str.savefile);

      fprintf (g_ergfile,"----------------------------------------------------------------\n");
      fprintf (g_ergfile,"number of round trips                 %ld\n", round_trips);
      fprintf (g_ergfile,"number of path inversions             %ld\n", ninvert);
      fprintf (g_ergfile,"transition time                       %lf\n", T_trans / n_trans);
      fprintf (g_ergfile,"TST energy                            %lf\n", g_TST_energy / g_TST_n);
      fprintf (g_ergfile,"number of transition states           %ld\n", N_TS);
      fprintf (g_ergfile,"number of analyzed paths              %ld\n", n_search);
      fprintf (g_ergfile,"transition states per path            %lf\n", ((REAL) N_TS) / n_search);

      /* print results */
      print_results ("AVRG", &avrg_str, i);

      /* Buffer flushen */
      fflush (NULL);

#ifdef XMOL
      fclose (g_xmolfile);
#endif

      elaps_time = time (NULL) - elaps_time;

      fprintf (g_ergfile,"----------------------------------------------------------------\n\n");

      fprintf (g_ergfile,"CPU-Time(sec)            %lf\n", cputime ());
      fprintf (g_ergfile,"CPU-Time                 %s\n", str_time (cpu_time = cputime (), tstr));
      fprintf (g_ergfile,"Elapsed_time             %s\n", str_time ((REAL) elaps_time, tstr));

#ifdef GRAPHICS
      close_graphics ();
#endif

      exit (0);

      /* Dummy for Borland C++ */
      return (0);
}
