/* *FH*******************************************************************  */
/* *                                                                    *  */
/* * FILE-NAME     : diat_ps.h                                          *  */
/* * FUNCTION      : Constants for diat_ps.c                            *  */
/* *                                                                    *  */
/* * AUTHOR        : Ch. Dellago                                        *  */
/* * CREATION-DATE : 12-02-1998                                         *  */
/* *                                                                    *  */
/* **********************************************************************  */

/*-------------------------------------------------------------------------*/
/*              COMPILER SWITCHES                                          */
/*-------------------------------------------------------------------------*/

#define  GRAPHICS               /* with graphics                           */
/* #define  END_IN_B */         /* Path ensemble                           */
/* #define  CANONICAL */        /* Canonical or microcanonical             */
/* #define  XMOL */             /* output in xmol-format                   */
/* #define  TSE */              /* Transition state ensemble               */

/*-------------------------------------------------------------------------*/
/*              General constants                                          */
/*-------------------------------------------------------------------------*/

#define  DIM      2                        /* Dimension                    */

#define  NSHOTS      1000
#define  NSHOTS_MIN  10
#define  NSHOTS_MAX  1000
#define  SHOT_STEPS  100
#define  TS_TOLERANCE 1.5

#define  N_TS_MAX   20

#define  COLD       0
#define  WARM       1

#define  NAVERAGE   2 
#define  NRESCALE   1000

#define  ACCEPTED     0
#define  REJECTED     1
#define  NOT_ACCEPTED 1

#define  STATE_A      0.0
#define  STATE_B      1.0
#define  STATE_0      2.0

#define  FORWARD      0
#define  BACKWARD     1

#define  EPSILON  1.0e-5

#define  IA       16807                    /* random number generator */
#define  IM       2147483647
#define  AM       (1.0/IM)
#define  IQ       127773
#define  IR       2836
#define  MASK     123459876

#define  ESC      0x1b                  
#define  TRUE     1
#define  FALSE    0
#define  PI       3.14159265358979323846
#define  ON       1
#define  OFF      0
#define  LN2      0.693147180559945309417
#define  CFIT     1.1257

/* constants for function next_char */
#define  CODE     0
#define  COMM     1
#define  TO_CODE  2
#define  TO_COMM  3

#define  YES      1
#define  NO       0

#define  START    0
#define  STOP     1

#define  OUTSIDE  0
#define  INSIDE   1

/*-------------------------------------------------------------------------*/
/*              TYPE DEFINITIONS                                           */
/*-------------------------------------------------------------------------*/

typedef struct {DWORD  ncycles;
                DWORD  nrelax;
		REAL   CPU_secs;
                DWORD  ntau;
                DWORD  nio;
		DWORD  nTS;
		WORD   start;
                DWORD  navrgout;
                DWORD  ngraph;
                DWORD  npart;
		DWORD  max_shift;
		WORD   substeps;
                REAL   temp;
                REAL   timestep;
		REAL   R_A;
		REAL   R_B;
		REAL   R_C;
		REAL   delta_p;
		REAL   width;
		REAL   height;
		REAL   rho;
		REAL   shots;
		char   ergfile[50];
		char   filename[50];
		char   savefile[50];
		char   getfile[50];
		char   xmolfile[50];} INPUT_STR;

typedef struct {REAL  *epot;
                REAL  *epot2;
                REAL  *ekin;
                REAL  *ekin2;
                REAL  *r;
		REAL  *F;
		REAL  *h_B;
		REAL  *hdot_B;
		DWORD *seot_rej;
		DWORD *shot_rej;
		DWORD *shot_acc;
		REAL   dE;
		DWORD  nA;
		DWORD  nB;
                DWORD  navrg;} AVRG_STR;

