/* **********************************************************************  */
/* * graphics.c                                                         *  */
/* **********************************************************************  */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "odeinty.h"
#include "graphics.h"
#include "diat_ps.h"
#include "Xpgp.h"

/* #define POSTSCRIPT */
#define PSFILE        "out.ps"

/*------------------------------------------------------------------------*/

extern DWORD         g_collision;
extern REAL          g_rho;
extern REAL          g_energy;
extern REAL          g_stepsize;
extern REAL          g_R_A, g_R_B;

extern REAL          g_box_size[DIM];
extern WORD          g_npart;
extern DWORD         g_nworm;
extern WORD          g_n_lya;
extern WORD          g_last_disk_1;
extern WORD          g_last_disk_2;
extern REAL         *g_charge;
extern REAL          g_EE[DIM];
extern REAL          g_state;

DWORD  g_last_coll;
REAL  *g_yold;
REAL  *g_lold;
REAL   g_ttime;

/*------------------------------------------------------------------------*/

#define NCOLOR  5
WORD  g_nparticles;
REAL  g_xmin, g_xmax;
REAL  g_ymin, g_ymax;
REAL  g_oxmin, g_oxmax;
REAL  g_oymin, g_oymax;
REAL  g_x_length, g_y_length;
WORD  g_radius;
WORD  g_x_zero, g_y_zero;
WORD  g_windowy;
WORD  g_color[NCOLOR];
int   g_disk_win;
int   g_r_win;
int   g_erase;


/*------------------------------------------------------------------------*/

void convert_point (REAL  x,
                    REAL  y,
                    WORD  *nx,
                    WORD  *ny)
{
    *nx =             (WORD) ((x - g_xmin) / (g_xmax - g_xmin) * WINDOWX);
    *ny = g_windowy - (WORD) ((y - g_ymin) / (g_ymax - g_ymin) * g_windowy);

#ifdef POSTSCRIPT
    *nx += 100;
    *ny += 150;
#endif

}

/*------------------------------------------------------------------------*/

void convert_opoint (REAL  x,
                     REAL  y,
                     WORD  *nx,
                     WORD  *ny)
{
    *nx =         (WORD) ((x - g_oxmin) / (g_oxmax - g_oxmin) * RWINX);
    *ny = RWINY - (WORD) ((y - g_oymin) / (g_oymax - g_oymin) * RWINY);
}


/*------------------------------------------------------------------------*/

void init_graphics (REAL   xmin,
                    REAL   xmax,
                    REAL   ymin,
                    REAL   ymax)
{
    /* Globale Variable versorgen */
    g_xmin = xmin;
    g_xmax = xmax;
    g_ymin = ymin;
    g_ymax = ymax;

    g_oxmin = 0.0;
    g_oxmax = 1.0;
    g_oymin = 0.5;
    g_oymax = 2.0;

    g_x_length = xmax - xmin;
    g_y_length = ymax - ymin;
    g_windowy  = (WORD) ((g_ymax - g_ymin) / (g_xmax - g_xmin) * WINDOWX);

    g_radius = (WORD) (DISK_RADIUS * WINDOWX) / g_x_length;

    /* Buffer allocieren */
    if ((g_yold = (REAL *) malloc (sizeof (REAL) * 4 * g_npart)) == NULL)
    {
        printf ("INIT_GRAPHICS> Fehler beim allokieren von %ld Bytes!!\n",
                sizeof (REAL) * 4 * g_npart);
        exit (0);
    }

    /* Buffer allocieren */
    if ((g_lold = (REAL *) malloc (sizeof (REAL) * 4 * g_npart)) == NULL)
    {
        printf ("INIT_GRAPHICS> Fehler beim allokieren von %ld Bytes!!\n",
                sizeof (REAL) * 4 * g_npart);
        exit (0);
    }

    /* Window oeffnen */
#ifdef POSTSCRIPT
    g_disk_win = OpenPS(PSFILE, PORT);
#else
    g_disk_win = OpenX(WINDOWX, g_windowy);
#endif

    g_r_win = OpenX(RWINX, RWINY);

    /* Farben */
    g_color[0] = 4;
    g_color[1] = 1;
    g_color[2] = 14;
    g_color[3] = 18;
    g_color[4] = 30;
}

/*------------------------------------------------------------------------*/

void draw_border (void)
{
    WORD nx, ny;
    WORD mx, my;
    WORD i;

    convert_point (g_xmin, g_ymin, &nx, &ny);
    convert_point (g_xmax, g_ymax, &mx, &my);

    fRectangleS(nx, ny, mx, my);
}

/*------------------------------------------------------------------------*/

void draw_oline (REAL xfrom,
                 REAL yfrom,
                 REAL xto,
                 REAL yto)
{
    WORD nx, ny;
    WORD mx, my;
    WORD i;

    convert_opoint (xfrom, yfrom, &nx, &ny);
    convert_opoint (xto, yto, &mx, &my);

    fLine (nx, ny, mx, my);
}

/*------------------------------------------------------------------------*/

void draw_arrow (REAL  x,
                 REAL  y,
                 REAL  px,
                 REAL  py)
{
    WORD nx, ny;
    WORD mx, my;
    REAL ax, ay;
    REAL ex, ey;
    REAL rx, ry;

    rx = px / 2.0;
    ry = py / 2.0;

    convert_point (x,    y,    &nx, &ny);
    convert_point (x+rx, y+ry, &mx, &my);

    fLine (nx, ny, mx, my);

    ax = -px / sqrt(px*px + py*py) * ARROW_LENGTH;
    ay = -py / sqrt(px*px + py*py) * ARROW_LENGTH;

    ex = x + rx + cos (ARROW_ANGLE) * ax - sin (ARROW_ANGLE) * ay;
    ey = y + ry + sin (ARROW_ANGLE) * ax + cos (ARROW_ANGLE) * ay;

    convert_point (ex, ey, &nx, &ny);

    fLine (nx, ny, mx, my);

    ex = x + rx + cos (ARROW_ANGLE) * ax + sin (ARROW_ANGLE) * ay;
    ey = y + ry - sin (ARROW_ANGLE) * ax + cos (ARROW_ANGLE) * ay;

    convert_point (ex, ey, &nx, &ny);

    fLine (nx, ny, mx, my);
}

/*------------------------------------------------------------------------*/

void draw_opoint (REAL  x,
                  REAL  y)
{
    WORD nx, ny;

    convert_opoint (x, y, &nx, &ny);
    fFilledCircle (nx, ny, 3);
}

/*------------------------------------------------------------------------*/

void draw_circle (REAL  x,
                  REAL  y,
                  REAL  px,
                  REAL  py,
                  REAL  r,
                  REAL  t,
                  WORD  c)
{
    WORD nx, ny;
    WORD i;
    WORD ncolor, pcolor;

        Pen(c);

        convert_point (x, y, &nx, &ny);
        fCircle (nx, ny, r);

	if (x > g_box_size[0]-1.0)
	{
            convert_point (x-g_box_size[0], y, &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x < 1.0)
	{
            convert_point (x+g_box_size[0], y, &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (y > g_box_size[1]-1.0)
	{
            convert_point (x, y-g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (y < 1.0)
	{
            convert_point (x, y + g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x > g_box_size[0]-1.0 && y > g_box_size[1]-1.0)
	{
            convert_point (x-g_box_size[0], y - g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x > g_box_size[0]-1.0 && y < 1.0)
	{
            convert_point (x-g_box_size[0], y + g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x < 1.0 && y > g_box_size[1] - 1.0)
	{
            convert_point (x+g_box_size[0], y - g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}
	if (x < 1.0 && y < 1.0)
	{
            convert_point (x+g_box_size[0], y + g_box_size[1], &nx, &ny);
            fCircle (nx, ny, r);
	}

        /* draw_arrow (x, y, px, py); */
        /*
        if (ch < 0.0)
           fLine (nx - CROSS/2, ny, nx + CROSS/2, ny);
        else
        {
           fLine (nx - CROSS/2, ny, nx + CROSS/2, ny);
           fLine (nx, ny - CROSS/2, nx, ny + CROSS/2);
        }
        */
}
/*------------------------------------------------------------------------*/

void draw_disks (REAL  *y,
                 REAL   o,
                 WORD   ndisks,
                 WORD   n)
{
     WORD  i, k;
     WORD  nx, ny;
     WORD  mx, my;
     static DWORD count=0;
     static REAL  last_time ;
     WORD   col;

     OutputTo (g_r_win);
     if (g_state == STATE_A) Pen(31); 
     else if (g_state == STATE_B) Pen(4);
     else Pen(14);
     draw_opoint (((REAL) n) / (g_nworm-1), o);
     fShow ();

     OutputTo (g_disk_win);
     SetLineWidth(2);

#ifndef POSTSCRIPT
     if (count != 0)
     {
         for (i = 0; i < ndisks; i++)
             draw_circle (g_yold[i*2], g_yold[i*2+1],
                          g_yold[(i+g_npart)*2], g_yold[(i+g_npart)*2+1],g_radius, last_time, 0);
     }
#endif

     if (g_state == STATE_A)
	 col = 31; 
     else if (g_state == STATE_B)
	 col = 4;
     else
	 col = 14;

     for (i = 0; i < ndisks; i++)
     { 
	 if (i == 0 || i == 1)
         draw_circle (y[i*2], y[i*2 + 1], y[(i+g_npart)*2], y[(i+g_npart)*2 + 1],g_radius, g_ttime, col);
	 else
         draw_circle (y[i*2], y[i*2 + 1], y[(i+g_npart)*2], y[(i+g_npart)*2 + 1],g_radius, g_ttime, 1);
     }

     /* Vektor speichern */
     count++;
     last_time = g_ttime;
     memcpy ((void *) g_yold, (void *) y, 4 * g_npart * sizeof (REAL));

     fShow();

#ifdef POSTSCRIPT
     draw_border ();
     Clear ();
#endif
}


/*------------------------------------------------------------------------*/

void draw_order (REAL *o,
                 WORD  n)
{
     WORD  i, k;
     WORD  nx, ny;
     WORD  col;
     REAL  dx;

     dx = 1.0 / (n - 1);

     OutputTo (g_r_win);
     Clear ();
     SetLineWidth(1);

     Pen(31);
     draw_oline (0.0, g_R_A, 1.0, g_R_A);
     Pen(4);
     draw_oline (0.0, g_R_B, 1.0, g_R_B);

     /*
     Pen(5);
     draw_oline (0.0, 1.0, 1.0, 1.0);
     draw_oline (0.0, 1.5, 1.0, 1.5);
     */

     Pen(1);
     SetLineWidth(1);
     for (i = 1; i < g_nworm; i++)
        draw_oline ((i-1) * dx, o[i-1], i * dx, o[i]);

     fShow();
}

/*------------------------------------------------------------------------*/

void close_graphics ()
{
}

